﻿const Dashboard = {

    defaultChartColors: ['rgb(153, 102, 255)', 'rgb(54, 162, 235)', 'rgb(75, 192, 192)', 'rgb(201, 203, 207)',
        'rgb(255, 99, 132)', 'rgb(255, 159, 64)', 'rgb(255, 205, 86)'],

    HubConnection: null,

    DashboardDataSourceSettings: {},

    DashboardSettings: {},
    Charts: [],

    InitializeDashboards: function (dashboardSettings) {
        this.DashboardSettings = dashboardSettings;

        let dashboardCount = dashboardSettings.DashboardData.length;

        let chartPlaceholder = $(".chart-placeholder");

        for (let i = 0; i < dashboardCount; i++) {
            let brickDiv = this.FormBrickDiv(i + 1, dashboardSettings.DashboardData[i].Name);
            chartPlaceholder.before(brickDiv);
        }

        for (let dashboardData of dashboardSettings.DashboardData) {
            this.CallFormChartData(dashboardData);
        }
    },

    CreatePieChart: function (ctx) {
        var config = {
            type: "pie",
            options: {
                responsive: true,
                maintainAspectRatio: false
            }
        };

        return new Chart(ctx, config);
    },

    CreateBarChart: function (ctx) {
        var options =
        {
            scaleGridLineWidth: 1,
            scaleFontSize: 10,
            scaleShowHorizontalLines: false,
            scaleShowVerticalLines: false,

            legend: { display: false },

            maintainAspectRatio: false,
            scales: {
                xAxes: [{
                    stacked: true,
                    gridLines: {
                        display: false
                    }
                }],
                yAxes: [{
                    stacked: true,
                    ticks: {
                        beginAtZero: true
                    },
                    gridLines: {
                        display: true,
                        borderDash: [5, 6]
                    },
                    scaleLabel: {
                        display: true
                    }
                }]
            },
        };

        return new Chart(ctx, {
            options: options,
            type: 'bar'
        });
    },

    CreateLineChart: function (ctx) {
        let options = {
            responsive: true,
            maintainAspectRatio: false,
            plugins: {
                legend: {
                    position: 'top',
                }
            }
        };

        return new Chart(ctx, {
            options: options,
            type: 'line'
        });
    },

    UpdateCanvas: function (name, position, chartMode, labels, chartData, colors, yAxisLabel) {

        let canvasElement = $('canvas[data-position="' + position + '"]');

        let canvasContext = canvasElement.get(0).getContext("2d");

        this.DestroyExistingChart(position);

        let colorData = this.defaultChartColors;

        if (colors && colors.length > 0)
            colorData = colors;

        // Pie
        if (chartMode == 0) {
            let pieChart = this.CreatePieChart(canvasContext);

            let diagramData = this.GetPieChartDataset(chartData, colorData);

            pieChart.data = diagramData;
            pieChart.update();

            this.Charts.push(pieChart);
        }
        // Bar
        else if (chartMode == 1) {
            let barChart = this.CreateBarChart(canvasContext);

            let barChartData = this.GetBarChartDataset(chartData, labels, colorData);

            if (yAxisLabel)
                barChart.options.scales.yAxes[0].scaleLabel.labelString = yAxisLabel;

            barChart.data = barChartData;
            barChart.update();

            this.Charts.push(barChart);
        }
        // Line
        else if (chartMode == 2) {
            let lineChart = this.CreateLineChart(canvasContext);

            let lineChartData = this.GetLineChartDataset(chartData, labels, colorData);

            lineChart.data = lineChartData;
            lineChart.update();

            this.Charts.push(lineChart);
        }

        let chartNameSpanElement = canvasElement.closest("div.brick").find('span[data-purpose="chartName"]');
        chartNameSpanElement.text(name);
    },

    GetPieChartDataset: function (chartData, colorData) {
        let pieData = [];

        for (let key in chartData) {
            let dataset = chartData[key];

            let sum = dataset.reduce(function (a, b) {
                return a + b;
            }, 0);

            pieData.push(sum);
        }

        let diagramData = {
            datasets:
                [{
                    data: pieData,
                    backgroundColor: colorData,
                    borderWidth: 1
                }],
            labels: Object.keys(chartData)
        }

        return diagramData;
    },

    GetBarChartDataset: function (chartData, labels, colorData) {
        let i = 0;
        let chartColorsCount = colorData.length;
        let chartDatasets = [];

        for (let key in chartData) {
            let record = chartData[key];

            let color = colorData[i % chartColorsCount];

            let dataset = {
                label: key,
                data: record,
                backgroundColor: color,
                borderWidth: 1
            };

            chartDatasets.push(dataset);

            i += 1;
        }

        let barChartData =
        {
            labels: labels,
            datasets: chartDatasets
        };

        return barChartData;
    },

    GetLineChartDataset: function (chartData, labels, colorData) {
        let i = 0;
        let chartColorsCount = colorData.length;
        let chartDatasets = [];

        for (let key in chartData) {
            let record = chartData[key];

            let color = colorData[i % chartColorsCount];

            let dataset = {
                label: key,
                data: record,
                borderColor: color,
                backgroundColor: color,
                fill: false,
                tension: 0
            };

            chartDatasets.push(dataset);

            i += 1;
        }

        let lineChartData = {
            labels: labels,
            datasets: chartDatasets
        };

        return lineChartData
    },

    DestroyExistingChart: function (position) {
        let existingChartIndex = this.Charts.findIndex(f => { return f.canvas.dataset.position == position });
        let existingChart = this.Charts[existingChartIndex];

        if (existingChart) {
            existingChart.destroy();
            this.Charts.splice(existingChartIndex, 1);
        }
    },

    ShowChartParametersMenu: function (menuId, position) {
        let chartParametersMenuElement = $("#" + menuId);
        chartParametersMenuElement.modal("show");

        let canvasPositionElement = $("#canvasPosition");
        canvasPositionElement.val(position);

        if (this.DashboardSettings && this.DashboardSettings.DashboardData) {

            let targetSettings = this.DashboardSettings.DashboardData.find(f => { return f.Position == position });

            if (targetSettings) {

                $("#ChartName").val(targetSettings.Name);
                $("#ChartMode").val(targetSettings.ChartMode);
                $("#DataSource").val(targetSettings.DataSource);

                if (targetSettings.UseDynamicUpdate)
                    $("#UseDynamicUpdate").prop("checked", true);
                else
                    $("#UseDynamicUpdate").prop("checked", false);

                if (targetSettings.FilterData) {
                    let filterData = targetSettings.FilterData;

                    $("#ChartPeriod").val(filterData.ChartPeriod);

                    $("#PeriodStart").val(filterData.PeriodStart.substring(0, 10));
                    $("#PeriodEnd").val(filterData.PeriodEnd.substring(0, 10));

                    $("#licensesDataSelectList").val(filterData.LicenseIds);
                    $("#tasksDataSelectList").val(filterData.TaskIds);
                    $("#queuesDataSelectList").val(filterData.QueueId);
                    $("#taskQueuesDataSelectList").val(filterData.TaskQueueId);

                    multipleSelects.forEach(element => element.multipleSelect("refresh"));
                }
            }
        }

        let dataSourceElement = document.getElementById("DataSource");
        this.OnDataSourceChanged(dataSourceElement);

        if (dataSourceElement.value != 2 && dataSourceElement.value != 3) {
            let chartPeriodElement = document.getElementById("ChartPeriod");
            this.OnChartPeriodChanged(chartPeriodElement);
        }
    },

    ApplyChartParameters: function (menuId) {

        let chartName = $("#ChartName").val();

        let position = $("#canvasPosition").val();

        let chartMode = $("#ChartMode").val();
        let dataSource = $("#DataSource").val();

        let useDynamicUpdate = $("#UseDynamicUpdate").prop("checked");

        let chartPeriod = $("#ChartPeriod").val();

        let periodStart = $("#PeriodStart").val();
        let periodEnd = $("#PeriodEnd").val();

        let licenseIds = $("#licensesDataSelectList").val();
        let taskIds = $("#tasksDataSelectList").val();
        let queueId = $("#queuesDataSelectList").val();
        let taskQueueId = $("#taskQueuesDataSelectList").val();

        let filterData = {
            ChartPeriod: chartPeriod,
            PeriodStart: periodStart,
            PeriodEnd: periodEnd,
            LicenseIds: licenseIds,
            TaskIds: taskIds,
            QueueId: queueId,
            TaskQueueId: taskQueueId
        };

        let dashboardData = {
            Name: chartName,
            Position: position,
            ChartMode: chartMode,
            DataSource: dataSource,
            FilterData: filterData,
            UseDynamicUpdate: useDynamicUpdate
        }

        if (this.DashboardSettings && this.DashboardSettings.DashboardData) {
            let targetSettingsIndex = this.DashboardSettings.DashboardData.findIndex(f => { return f.Position == position });
            let targetSettings = this.DashboardSettings.DashboardData[targetSettingsIndex];

            if (targetSettings) {
                this.DashboardSettings.DashboardData[targetSettingsIndex] = dashboardData;
            }
            else {
                this.DashboardSettings.DashboardData.push(dashboardData);
            }
        }

        if (position > this.Charts.length) {
            let canvasSelector = $('canvas[data-position="' + position + '"');
            let chartPlaceholder = $(".chart-placeholder");

            if (canvasSelector.length === 0) {
                let brickDiv = this.FormBrickDiv(position, chartName);

                chartPlaceholder.before(brickDiv);
            }
        }

        this.CallFormChartData(dashboardData);
        this.CallSaveDashboardSettings(this.DashboardSettings);

        if (useDynamicUpdate)
            this.SubscribeToDataUpdates();

        let chartParametersMenuElement = $("#" + menuId);
        chartParametersMenuElement.modal("hide");
    },

    DeleteChart: function (position) {
        this.DestroyExistingChart(position);

        let canvasElement = $('canvas[data-position="' + position + '"');
        canvasElement.parent().parent().remove();

        let recountedCanvases = $("canvas[data-position]").filter(function () {
            return parseInt($(this).attr("data-position")) > position;
        });

        recountedCanvases.each(function () {
            let currentPosition = $(this).attr("data-position");
            $(this).attr("data-position", currentPosition - 1);
        })

        let chartPlaceholder = $(".chart-placeholder");

        if (chartPlaceholder.length === 0) {
            let chartPlaceholderElement = this.FormChartPlaceholder();
            $("#ChartSection").append(chartPlaceholderElement);
        }

        let existingDashboardDataIndex = this.DashboardSettings.DashboardData.findIndex(f => { return f.Position == position });
        let existingDashboardData = this.DashboardSettings.DashboardData[existingDashboardDataIndex];

        if (existingDashboardData) {
            this.DashboardSettings.DashboardData.splice(existingDashboardDataIndex, 1);

            let dashboardDataLength = this.DashboardSettings.DashboardData.length;

            if (existingDashboardDataIndex < dashboardDataLength) {
                for (let i = existingDashboardDataIndex; i < dashboardDataLength; i++) {
                    this.DashboardSettings.DashboardData[i].Position -= 1;
                }
            }
        }

        this.CallSaveDashboardSettings();
    },

    FormBrickDiv: function (position, chartName) {

        let brickDiv = document.createElement("div");
        brickDiv.className = "brick";

        brickDiv.ondragover = function (ev) {
            ev.preventDefault();
            ev.target.style.opacity = 0.6;
        };

        brickDiv.ondragleave = function (ev) {
            ev.target.style.opacity = 1;
        };

        brickDiv.ondrop = function (ev) {
            let dataSource = ev.dataTransfer.getData("source");

            if (dataSource === "diagram") {
                let sourcePosition = ev.dataTransfer.getData("position");
                let sourceChart = Dashboard.DashboardSettings.DashboardData.find(f => f.Position == sourcePosition);
                let sourceCanvas = $("canvas[data-position='" + sourcePosition + "']")[0];

                let targetPosition = $(ev.target).closest("div.brick").find("canvas[data-position]").attr("data-position");
                let targetChart = Dashboard.DashboardSettings.DashboardData.find(f => f.Position == targetPosition);
                let targetCanvas = $("canvas[data-position='" + targetPosition + "']")[0];

                sourceChart.Position = targetPosition;
                targetChart.Position = sourcePosition;

                sourceCanvas.dataset["position"] = targetPosition;
                targetCanvas.dataset["position"] = sourcePosition;

                let sourceBrick = sourceCanvas.closest("div.brick");
                let targetBrick = targetCanvas.closest("div.brick");

                let targetBrickCopy = targetBrick.cloneNode(true);

                sourceBrick.parentNode.insertBefore(targetBrickCopy, sourceBrick);
                targetBrick.parentNode.insertBefore(sourceBrick, targetBrick);
                targetBrick.parentNode.replaceChild(targetBrick, targetBrickCopy);

                Dashboard.CallSaveDashboardSettings();

                ev.target.style.opacity = 1;
            }
        };

        let chartHeader = document.createElement("h4");
        chartHeader.className = "chart-sign";
        chartHeader.draggable = true;
        chartHeader.style.cursor = "move";

        chartHeader.ondragstart = function (ev) {
            let currentPosition = $(ev.target).closest("div.brick").find("canvas[data-position]").attr("data-position");
            ev.dataTransfer.setData("position", currentPosition);
            ev.dataTransfer.setData("source", "diagram");
        };

        let chartNameSpan = document.createElement("span");
        chartNameSpan.dataset.purpose = "chartName";
        chartNameSpan.innerText = chartName;
        chartHeader.appendChild(chartNameSpan);

        let chartButtonsSpan = document.createElement("span");
        chartButtonsSpan.style.float = "right";

        let chartOptionsButtonElement = document.createElement("button");
        chartOptionsButtonElement.className = "btn btn-outline-dark btn-sm";
        chartOptionsButtonElement.innerHTML = "&#9881;";
        chartOptionsButtonElement.onclick = function () {
            let currentPosition = $(this).closest("div.brick").find("canvas[data-position]").attr("data-position");
            Dashboard.ShowChartParametersMenu('chartParameters', currentPosition);
        };

        let deleteChartButtonElement = document.createElement("button");
        deleteChartButtonElement.className = "btn btn-outline-dark btn-sm";
        deleteChartButtonElement.style.marginLeft = "10px";
        deleteChartButtonElement.innerHTML = "&times;";
        deleteChartButtonElement.onclick = function () {
            let currentPosition = $(this).closest("div.brick").find("canvas[data-position]").attr("data-position");
            Dashboard.DeleteChart(currentPosition);
        };

        chartButtonsSpan.appendChild(chartOptionsButtonElement);
        chartButtonsSpan.appendChild(deleteChartButtonElement);

        chartHeader.appendChild(chartButtonsSpan);
        brickDiv.appendChild(chartHeader);

        let hrElement = document.createElement("hr");
        brickDiv.appendChild(hrElement);

        let chartDiv = document.createElement("div");
        chartDiv.className = "chart";

        let canvasElement = document.createElement("canvas");
        canvasElement.dataset.position = position;

        chartDiv.appendChild(canvasElement);
        brickDiv.appendChild(chartDiv);

        return brickDiv;
    },

    FormChartPlaceholder: function () {
        let chartPlaceholderDiv = document.createElement("div");
        chartPlaceholderDiv.className = "brick chart-placeholder btn";
        chartPlaceholderDiv.onclick = function () {
            Dashboard.ShowChartParametersMenu('chartParameters', Dashboard.Charts.length + 1);
        }

        let innerSpan = document.createElement("span");
        innerSpan.innerText = "+";

        chartPlaceholderDiv.appendChild(innerSpan);

        return chartPlaceholderDiv;
    },

    CallFormChartData: function (dashboardData) {
        $.ajax({
            url: "/Dashboard/FormChartData",
            contentType: "application/x-www-form-urlencoded",
            method: "POST",
            data: {
                dashboardData: dashboardData
            },
            success: function (response) {
                Dashboard.UpdateCanvas(dashboardData.Name, dashboardData.Position, dashboardData.ChartMode, response.labels, response.datasets, response.colors, response.yAxisLabel);
            }
        });
    },

    CallSaveDashboardSettings: function () {
        $.ajax({
            url: "/Dashboard/SaveDashboardSettings",
            contentType: "application/x-www-form-urlencoded",
            method: "POST",
            data: {
                dashboardSettings: Dashboard.DashboardSettings
            },
            success: function (response) {
                console.log("dashboard settings were successfully saved");
            }
        });
    },

    OnChartModeChanged: function (chartModeElement) {

    },

    OnDataSourceChanged: function (dataSourceElement) {
        let licensesDataGroupElement = $("#LicensesDataGroup");
        let tasksDataGroupElement = $("#TasksDataGroup");
        let queuesDataGroupElement = $("#QueuesDataGroup");
        let taskQueuesDataGroupElement = $("#TaskQueuesDataGroup");

        let chartPeriodRowElement = $("#ChartPeriodGroup");
        let periodRowElement = $("#PeriodGroup");

        let dataSource = dataSourceElement.value;

        licensesDataGroupElement.hide();
        tasksDataGroupElement.hide();
        queuesDataGroupElement.hide();
        taskQueuesDataGroupElement.hide();

        chartPeriodRowElement.hide();
        periodRowElement.hide();

        let targetDataSourceSettings = this.DashboardDataSourceSettings.find(obj => {
            return obj.DataSource == dataSource;
        });

        let availableFilters = targetDataSourceSettings.AvailableFilters;

        if (availableFilters) {
            // Chart period
            if (availableFilters.includes(0)) {
                chartPeriodRowElement.show();
                periodRowElement.show();
            }
            // Tasks
            if (availableFilters.includes(1)) {
                tasksDataGroupElement.show();
            }
            // Licenses
            if (availableFilters.includes(2)) {
                licensesDataGroupElement.show();
            }
            // Queues
            if (availableFilters.includes(3)) {
                queuesDataGroupElement.show();
            }
            // Task queues
            if (availableFilters.includes(4)) {
                taskQueuesDataGroupElement.show();
            }
        }

        let availableChartModes = targetDataSourceSettings.AvailableChartModes;

        if (availableChartModes && availableChartModes.length > 0) {
            let chartModeElement = $("#ChartMode");

            chartModeElement.children("option").hide();

            for (let chartModeValue of availableChartModes) {
                chartModeElement.children("option[value=" + chartModeValue + "]").show();
            }

            let currentChartModeValue = chartModeElement.val();

            if (!availableChartModes.includes(parseInt(currentChartModeValue)))
                chartModeElement.val(availableChartModes[0]);
        }
    },

    OnChartPeriodChanged: function (chartPeriodElement) {
        let periodGroupElement = $("#PeriodGroup");

        let chartPeriod = chartPeriodElement.value;

        // Time period
        if (chartPeriod == 0)
            periodGroupElement.show()
        else
            periodGroupElement.hide();
    },

    SubscribeToDataUpdates: function () {
        if (!Dashboard.HubConnection) {
            let hubUrl = '/dataevents';

            let hubConnection = new signalR.HubConnectionBuilder()
                .withUrl(hubUrl)
                .build();

            hubConnection.on('TaskHistoryUpdate', function (id) {
                Dashboard.UpdateDynamicDashboardsByDataSource(0);
                Dashboard.UpdateDynamicDashboardsByDataSource(1);
                Dashboard.UpdateDynamicDashboardsByDataSource(4);
                Dashboard.UpdateDynamicDashboardsByDataSource(6);
            });

            hubConnection.on('AgentStatusUpdate', function (id, name, isConnect) {
                Dashboard.UpdateDynamicDashboardsByDataSource(2);
            });

            hubConnection.on('SchedulerUpdate', function (id) {
                Dashboard.UpdateDynamicDashboardsByDataSource(3);
            });

            hubConnection.on('QueuesUpdate', function (id) {
                Dashboard.UpdateDynamicDashboardsByDataSource(5);
            });

            hubConnection.on('TaskQueuesUpdate', function () {
                Dashboard.UpdateDynamicDashboardsByDataSource(7);
            });

            hubConnection.start().then(() => {
                hubConnection.invoke("Subscribe", "Tasks");
                hubConnection.invoke("Subscribe", "Agents");
                hubConnection.invoke("Subscribe", "Schedulers");
                hubConnection.invoke("Subscribe", "Queues");
                hubConnection.invoke("Subscribe", "TaskQueues");
            });

            Dashboard.HubConnection = hubConnection;
        }
    },

    UpdateDynamicDashboardsByDataSource: function (dataSource) {
        let targetDashboards = Dashboard.DashboardSettings.DashboardData.filter(f => f.DataSource == dataSource && f.UseDynamicUpdate);

        for (let td of targetDashboards) {
            $.ajax({
                url: "/Dashboard/FormChartData",
                contentType: "application/x-www-form-urlencoded",
                method: "POST",
                data: {
                    dashboardData: td
                },
                success: function (response) {
                    let position = td.Position;
                    let existingChartIndex = Dashboard.Charts.findIndex(f => { return f.canvas.dataset.position == position });
                    let existingChart = Dashboard.Charts[existingChartIndex];

                    let labels = response.labels;
                    let chartData = response.datasets;
                    let colors = response.colors;

                    let colorData = Dashboard.defaultChartColors;

                    if (colors && colors.length > 0)
                        colorData = colors;

                    let chartMode = td.ChartMode;

                    let diagramData = null;

                    if (chartMode == 0) 
                        diagramData = Dashboard.GetPieChartDataset(chartData, colorData);
                    else if (chartMode == 1)
                        diagramData = Dashboard.GetBarChartDataset(chartData, labels, colorData);
                    else if (chartMode == 2)
                        diagramData = Dashboard.GetLineChartDataset(chartData, labels, colorData);

                    existingChart.data = diagramData;
                    existingChart.update(0);
                }
            });
        }
    }
}