﻿var FilterPanel = {
    filterDivIndex: 0,

    localizedDropdownElements: {},

    ChangeFilterPanelVisibility: function (panelId) {
        var x = document.getElementById(panelId);
        if (x.style.display === "none") {
            x.style.display = "table-row";
        } else {
            x.style.display = "none";
        }
    },

    ParsingResult: function (inputValue, compareOperator) {
        this.InputValue = inputValue;
        this.CompareOperator = compareOperator;
    },

    addFilterCondition: function (filterDivId, modelDescription) {

        let panel = document.getElementById(filterDivId);

        let selectedField = document.getElementById("fieldsList");
        let fieldName = selectedField.options[selectedField.selectedIndex].value;
        let fieldDescription = modelDescription.find(item => item.FullName == fieldName);

        console.log(selectedField.options[selectedField.selectedIndex].value);

        let div = this.addDiv(fieldName);

        let label = this.addLabel(fieldDescription.DisplayName);

        this.CreateInput(div, fieldDescription);

        let deleteButton = this.addDeleteButton(div.id);

        let fullNameInput = document.createElement("input");
        fullNameInput.type = "hidden";
        fullNameInput.id = "FullName";
        fullNameInput.value = fieldDescription.FullName;

        let displayNameInput = document.createElement("input");
        displayNameInput.type = "hidden";
        displayNameInput.id = "DisplayName";
        displayNameInput.value = fieldDescription.DisplayName;

        let typeNameInput = document.createElement("input");
        typeNameInput.type = "hidden";
        typeNameInput.id = "TypeName";
        typeNameInput.value = fieldDescription.TypeName;

        let enumNamesInput = document.createElement("input");
        enumNamesInput.type = "hidden";
        enumNamesInput.id = "EnumNames";
        enumNamesInput.value = fieldDescription.EnumNames;

        panel.appendChild(div);

        div.prepend(fullNameInput);
        div.prepend(displayNameInput);
        div.prepend(typeNameInput);
        div.prepend(enumNamesInput);

        div.prepend(label);
        div.appendChild(deleteButton);
    },

    addDeleteButton: function (divId) {
        let deleteBut = document.createElement('button');

        deleteBut.className = "button del_button filter_field ";
        deleteBut.onclick = () => {
            this.deleteFilterField(divId);
        };
        deleteBut.innerText = "X";

        return deleteBut;
    },

    addDiv: function (fieldName, type = undefined, filterPanel = undefined) {
        let div = document.createElement('div');
        div.className = "filter_field_div";

        if (type === 'logicOperator') {
            let lastElement = filterPanel.lastElementChild;
            let filterElementName = lastElement.id;

            div.setAttribute('name', filterElementName);
        }
        else {
            div.id = fieldName.replace(/\./g, "_") + "_" + this.filterDivIndex++;
            div.setAttribute('name', div.id);
            div.setAttribute('data-type', fieldName);
        }

        return div;
    },


    addLabel: function (fieldName) {
        let label = document.createElement('label');
        label.className = "filter_field condition_label";
        label.innerText = fieldName + " : ";

        return label;
    },

    CreateInput: function (div, fieldDescription, inputValue = undefined, compareOperator = undefined) {

        switch (fieldDescription.TypeName.toLowerCase()) {

            case "int32": this.createTextInput(div, "number", inputValue, fieldDescription.EnumNames);
                break;
            case "datetime": this.createDateInput(div, inputValue, compareOperator);
                break;
            case "enum": this.createDropDownList(div, fieldDescription.EnumNames, inputValue);
                break;
            case "collection":
                this.createDropDownList(div, fieldDescription.EnumNames, inputValue);
                break;
            case "bool":
                this.createCheckboxInput(div);
                break;
            default: this.createTextInput(div, "text", inputValue, fieldDescription.EnumNames);
                break;
        }
    },

    createDropDownList: function (div, optionsArray, selectedOption = undefined, setName = undefined, localizedElements = undefined) {

        let select = document.createElement('select');
        select.className = "input_field filter_field";
        select.id = div.id + "_value";

        if (setName != undefined) {
            select.name = setName;
            select.id = div.id + setName;
        }

        for (let name of optionsArray) {
            if (localizedElements && localizedElements[name])
                select.options.add(new Option(this.localizedDropdownElements[name], name));
            else
                select.options.add(new Option(name));
        }

        if (selectedOption != undefined)
            select.value = selectedOption; //Array.from(select.options).find(item => item.value === selectedOption).value;

        div.appendChild(select);

        return select;
    },

    createDateInput: function (div, inputValue = undefined, compareOperator = undefined) {

        let input = document.createElement('input');
        input.className = "input_field filter_field";
        input.type = "date";
        input.id = div.id + "_value";

        let compareOperList = document.createElement('select');
        compareOperList.id = div.id + "_compare";
        compareOperList.className = "input_field operation filter_field";
        compareOperList.options.add(new Option(">"));
        compareOperList.options.add(new Option("<"));
        compareOperList.options.add(new Option("="));

        if (inputValue != undefined && compareOperator != undefined) {
            input.value = inputValue;
            compareOperList.value = compareOperator;
        }

        div.appendChild(compareOperList);
        div.appendChild(input);

    },

    createTextInput: function (div, inputType, inputValue = undefined, dropdownValues = undefined) {

        let input = document.createElement('input');
        input.className = "input_field filter_field";
        input.type = inputType;
        input.id = div.id + "_value";

        if (inputValue != undefined)
            input.value = inputValue;

        if (inputType == "text") {
            let textOperatorSelectElement = this.createDropDownList(div, ['Contains', 'Equals', 'StartsWith', 'EndsWith'], 'Contains', 'textOperator', this.localizedDropdownElements);

            if (dropdownValues && dropdownValues.length > 0) {
                textOperatorSelectElement.setAttribute("onchange", "FilterPanel.changeTextInputsVisibility(this.value, '" + div.id + "')");

                let valuesDropdown = this.createDropDownList(div, dropdownValues, undefined, "_value_equals");
                valuesDropdown.style = "display: none;";
            }
        }

        div.appendChild(input);
    },

    createCheckboxInput: function (div) {
        let input = document.createElement("input");
        input.className = "check-box";
        input.type = "checkbox";
        input.id = div.id + "_value";

        div.appendChild(input);
    },

    deleteFilterField: function (id) {
        let field = document.getElementById(id);

        field.remove();

        if (this.filterDivIndex > 0)
            this.filterDivIndex--;
    },

    constructFilterRequest: function (divID, modelDescription) {

        let conditionPanel = document.getElementById(divID);
        let conditions = conditionPanel.childNodes;

        let queryOperands = [];

        for (let condition of conditions) {
            if (condition.id != undefined) {

                console.log(condition.getAttribute('name'));
                fieldInfo = modelDescription.find(item => item.FullName == condition.getAttribute('data-type'));

                let operand = this.constructOperand(condition, fieldInfo.FullName, fieldInfo.TypeName);

                if (operand && operand.length > 0) {
                    queryOperands.push(operand);
                    queryOperands.push("AND");
                }
                else {
                    let lastQueryOperand = queryOperands[queryOperands.length - 1];

                    if (lastQueryOperand && (lastQueryOperand == "AND" || lastQueryOperand == "OR"))
                        queryOperands.pop();
                }
            }
        }

        let firstQueryOperand = queryOperands[0];

        if (firstQueryOperand && (firstQueryOperand == "AND" || firstQueryOperand == "OR"))
            queryOperands = queryOperands.slice(1);

        let lastQueryOperand = queryOperands[queryOperands.length - 1];

        if (lastQueryOperand && (lastQueryOperand == "AND" || lastQueryOperand == "OR"))
            queryOperands.pop();

        let query = queryOperands.join(" ");

        return query;
    },

    constructOperand: function (condition, fieldName, fieldType) {
        let valueElement = document.getElementById(condition.id + "_value");
        let valueDropdownElement = document.getElementById(condition.id + "_value_equals");
        let compareElement = document.getElementById(condition.id + "_compare");
        let textOperator = Array.from(condition.childNodes)
            .find(item => item.name === 'textOperator');

        let value;

        if (valueDropdownElement && textOperator && textOperator.value === "Equals")
            value = this.getValueOfElement(valueDropdownElement, fieldType);
        else
            value = this.getValueOfElement(valueElement, fieldType);

        if (value && value.length > 0) {

            if (textOperator) {
                // Формат: [FieldName].[Contains|Equals|StartsWith|EndsWith]('searchText')
                return fieldName + '.' + textOperator.value + '(' + value + ')';
            }
            else if (fieldType.toLowerCase() === "collection") {
                return fieldName + '.Any(it == ' + value + ')';
            }
            else if (fieldType.toLowerCase() === "bool") {
                return fieldName + '.Equals(' + valueElement.checked + ')';
            }
            else {
                let compareOperator = "";

                if (compareElement)
                    compareOperator = compareElement.value;
                else
                    compareOperator = "=";

                // Формат: [FieldName] [=|>|<] 'searchText' 
                return fieldName + " " + compareOperator + " " + value;
            }
        }
        else
            return "";
    },

    getValueOfElement: function (item, fieldType) {
        if (item.value && item.value.length > 0) {
            if (item.tagName.toLowerCase() == "input" && item.type == "date") {
                return "DateTime(" + item.value.replace(new RegExp("-", 'g'), ",") + ")";
            }
            else {
                if (fieldType.toLowerCase() === "int32")
                    return item.value;
                else
                    return '"' + item.value + '"';
            }
        }
        else
            return "";
    },

    updateFilterCountElement: function () {
        let filterCountSpan = document.getElementById("filterCount");
        let buttonFilterCountSpan = document.getElementById("acceptFilterButtonFilterCounter");

        if (this.filterDivIndex > 0) {
            let counterText = "(" + FilterPanel.filterDivIndex + ")";
            filterCountSpan.innerHTML = counterText;
            buttonFilterCountSpan.innerHTML = counterText;
        }
        else {
            filterCountSpan.innerHTML = "";
            buttonFilterCountSpan.innerHTML = "";
        }
    },

    updateColumnFilterCountElement: function (columnName) {
        let filterCountSpan = document.getElementById(columnName + "_filter_count");

        let filterElementCount = document.querySelectorAll('[data-type="' + columnName + '"]').length;

        if (filterElementCount > 0)
            filterCountSpan.innerHTML = "(" + filterElementCount + ")";
        else
            filterCountSpan.innerHTML = "";
    },

    changeTextInputsVisibility: function (operatorValue, filterId) {
        let textInput = document.getElementById(filterId + "_value");
        let dropdownInput = document.getElementById(filterId + "_value_equals");

        if (operatorValue == "Equals") {
            textInput.style = "display: none;";
            dropdownInput.style = "";
        }
        else {
            textInput.style = "";
            dropdownInput.style = "display: none;";
        }
    },

    acceptFilter: function (controllerName, divID, fullNames, sortColumn, sortOrder, stringFields) {
        // Формируем параметры запроса
        let paramsString = "";

        // searchText
        let searchText = this.constructFilterRequest(divID, fullNames);
        //console.log(searchText);

        if (searchText != "")
            paramsString = paramsString + "filterText=" + searchText;

        // columnName
        if (sortColumn != "") {
            if (paramsString != "")
                paramsString = paramsString + "&";
            paramsString = paramsString + "sortColumnName=" + sortColumn;
        }

        // sortOrder
        if (sortOrder != "") {
            if (paramsString != "")
                paramsString = paramsString + "&";
            paramsString = paramsString + "sortType=" + sortOrder;
        }

        // fastFilterText
        let fastFilterValue = document.getElementById('fast-search-field').value.trim();

        if (fastFilterValue != "") {

            let fastFilterText = "";
            for (let i = 0; i < stringFields.length; i++) {
                fastFilterText += stringFields[i].FullName + '.Contains("' + fastFilterValue + '")' + (i + 1 < stringFields.length ? ' OR ' : '');
            }

            if (fastFilterText != "") {
                if (paramsString != "")
                    paramsString += "&";
                paramsString = paramsString + "fastFilterText=" + fastFilterText;
            }

            // Передаем строку для восстановления в поле fast-search-field
            if (paramsString != "")
                paramsString += "&";
            paramsString += "fastFilterValue=" + fastFilterValue;
        }

        // fastFilterPeriod
        let periodListElement = document.getElementById('period-list')

        if (periodListElement != null) {

            let fastFilterPeriod = periodListElement.options[periodListElement.selectedIndex].value;

            if (fastFilterPeriod != "" && fastFilterPeriod != "0") {
                if (paramsString != "")
                    paramsString += "&";
                paramsString += "fastFilterPeriod=" + fastFilterPeriod;
            }
        }

        // agentId
        let agentListElement = document.getElementById('agent-id')

        if (agentListElement != null) {

            let agent = agentListElement.options[agentListElement.selectedIndex].value;

            if (agent != "") {
                if (paramsString != "")
                    paramsString += "&";
                paramsString += "agent=" + agent;
            }
        }

        // Status
        let statusElement = document.getElementById('fast-search-status')

        if (statusElement != null) {

            let status = statusElement.options[statusElement.selectedIndex].value;

            if (status != "") {
                if (paramsString != "")
                    paramsString += "&";
                paramsString += "status=" + status;
            }
        }

        // Groups
        let groupsElement = document.getElementById('fast-search-groups')

        if (groupsElement != null) {

            let group = groupsElement.options[groupsElement.selectedIndex].value;

            if (group != "") {
                if (paramsString != "")
                    paramsString += "&";
                paramsString += "group=" + group;
            }
        }

        // processId
        let processesElement = document.getElementById('fast-search-processes')

        if (processesElement != null) {

            let processId = processesElement.options[processesElement.selectedIndex].value;

            if (processId != "") {
                if (paramsString != "")
                    paramsString += "&";
                paramsString += "processId=" + processId;
            }
        }

        // scheduleMode
        let scheduleModeElement = document.getElementById('fast-search-schedule-mode')

        if (scheduleModeElement != null) {

            let scheduleMode = scheduleModeElement.options[scheduleModeElement.selectedIndex].value;

            if (scheduleMode != "") {
                if (paramsString != "")
                    paramsString += "&";
                paramsString += "scheduleMode=" + scheduleMode;
            }
        }
        
        // Проверяем имеются ли данные
        if (paramsString != "")
            paramsString = "?" + paramsString;

        window.location.href = "/" + controllerName + paramsString;
    },

    clearFilters: function (controllerName) {
        window.location.href = "/" + controllerName
    },

    applyFilterSet: function (filterSetName, controllerName, filtersContainerId, fastFiltersContainerId, save, sortColumnName, sortType, pageNumber, duplicateFilterSetMessage) {

        if (filterSetName === "" && save == true) {
            filterSetName = document.getElementById('filterSetName').placeholder;
        }

        let filtersContainer = document.getElementById(filtersContainerId);
        let filters = filtersContainer.childNodes;

        let filterSetItems = [];

        for (let filter of filters) {
            if (filter.id != undefined) {

                let conditionElement = filter.querySelector("#" + filter.id + "_compare");
                if (conditionElement == null) conditionElement = filter.querySelector("#" + filter.id + "textOperator");

                let valueElement = filter.querySelector("#" + filter.id + "_value");
                if (valueElement.style.display === "none") {
                    valueElement = filter.querySelector("#" + filter.id + "_value_equals");
                }

                let value = null;

                if (valueElement.type === 'text' || valueElement.type === 'date' || valueElement.type === 'number') {
                    value = valueElement.value;
                } else if (valueElement.type === 'select-one') {
                    value = valueElement.options[valueElement.selectedIndex].text;
                } else if (valueElement.type === 'checkbox') {
                    value = valueElement.checked;
                }

                let filterSetItem = {
                    Id: filter.querySelector("#FilterId") != null ? filter.querySelector("#FilterId").value : null,
                    FullName: filter.querySelector("#FullName").value,
                    DisplayName: filter.querySelector("#DisplayName") != null ? filter.querySelector("#DisplayName").value : null,
                    TypeName: filter.querySelector("#TypeName") != null ? filter.querySelector("#TypeName").value : null,
                    EnumNames: filter.querySelector("#EnumNames") != null ? filter.querySelector("#EnumNames").value : null,
                    ConditionOperator: conditionElement != null ? conditionElement.options[conditionElement.selectedIndex].value : null,
                    Value: value
                }

                filterSetItems.push(filterSetItem);
            }
        }

        let fastFiltersContainer = document.getElementById(fastFiltersContainerId);
        let fastFilters = fastFiltersContainer.childNodes;

        let filterSetFastItems = [];

        for (let fastFilter of fastFilters) {
            if (fastFilter.name != undefined) {

                let value = "";

                if (fastFilter.type === "text") {
                    value = fastFilter.value;
                }

                if (fastFilter.type === "select-one" && fastFilter.childElementCount != 0) {
                    let selectedValue = fastFilter.options[fastFilter.selectedIndex].value;
                    value = !(selectedValue === 'default') ? selectedValue : "";
                }

                if (value != "") {

                    let filterSetFastItem = {
                        Id: fastFilter.id,
                        Name: fastFilter.name,
                        Value: value
                    }

                    filterSetFastItems.push(filterSetFastItem);
                }
            }
        }

        let currentFilterSetIdElement = document.getElementById('currentFilterSetId');
        let currentFilterSetId = currentFilterSetIdElement != null ? currentFilterSetIdElement.value : "";

        let urlParts = window.location.href.split('/');
        let applyFilterSetUrl = urlParts[0] + "//" + urlParts[2] + "/" + controllerName;
        if (save) {

            let filterSet;

            filterSet = {
                Id: currentFilterSetId,
                Name: filterSetName,
                ControllerName: controllerName
            }

            $.ajax({
                url: applyFilterSetUrl,
                contentType: "application/x-www-form-urlencoded",
                method: "POST",
                data: {
                    filterSet: filterSet,
                    filterSetItems: filterSetItems,
                    filterSetFastItems: filterSetFastItems,
                    save: save
                },
                success: function (response) {
                    if (response != -1) {
                        $("#createModal").modal("hide");
                        window.location.href = FilterPanel.constructUrlWithFilterSetId(response);
                    } else {
                        alert(duplicateFilterSetMessage);
                    }
                }
            });
        } else {

            let filterSet = {
                Id: currentFilterSetId,
                Name: filterSetName,
                ControllerName: controllerName
            }

            $.ajax({
                url: applyFilterSetUrl,
                contentType: "application/x-www-form-urlencoded",
                method: "POST",
                data: {
                    filterSet: filterSet,
                    filterSetItems: filterSetItems,
                    filterSetFastItems: filterSetFastItems,
                    save: save,
                    sortColumnName: sortColumnName,
                    sortType: sortType,
                    pageNumber: pageNumber,
                },
                success: function (response) {
                    document.open();
                    document.write(response);
                    document.close();
                }
            });
        }
    },

    showFilterSetCreateDialog: function (filterSetNamePrefix) {

        let currentFilterSetIdElement = document.getElementById('currentFilterSetId');
        let filterSetLabelElement = document.getElementById('filterSetLabel');

        let filterSetNameElement = document.getElementById("filterSetName");
        let filterSetIdToUpdateElement = document.getElementById('filterSetIdToUpdate');

        if (currentFilterSetIdElement != null) {
            filterSetNameElement.value = filterSetLabelElement.innerText;
            filterSetIdToUpdateElement.value = currentFilterSetIdElement.value;
        }

        let date = new Date();
        let dateString = date.getFullYear() + ("0" + (date.getMonth() + 1)).slice(-2) + ("0" + date.getDate()).slice(-2) + "_" + ("0" + date.getHours()).slice(-2) + ("0" + date.getMinutes()).slice(-2) + ("0" + date.getSeconds()).slice(-2);

        let defaultFilterSetName = filterSetNamePrefix + "_" + dateString;

        filterSetNameElement.placeholder = defaultFilterSetName;

        $("#createModal").modal("show");
    },

    showFilterSetDeleteDialog: function (filterSetId, filterSetName, message) {
        document.getElementById('filterSetIdToDelete').value = filterSetId;
        $("#deleteModalBody").text(message + " " + filterSetName + "?");
        $("#deleteModal").modal("show");
    },

    deleteFilterSet: function (filterSetId, controllerName) {

        let urlParts = window.location.href.split('/');
        let deleteUrl = urlParts[0] + "//" + urlParts[2] + "/FilterSets/Remove";

        $.ajax({
            url: deleteUrl,
            contentType: "application/x-www-form-urlencoded",
            method: "GET",
            data: {
                filterSetId: filterSetId,
                actionSection: controllerName
            },
            success: function () {

                $("#deleteModal").modal("hide");

                let currentFilterSetIdElement = document.getElementById('currentFilterSetId');

                if (currentFilterSetIdElement != null && currentFilterSetIdElement.value == filterSetId) {
                    window.location.href = window.location.href.split('?')[0];
                } else {
                    document.getElementById('filterSetRow_' + filterSetId).remove();
                }
            },
            error: function (error) {
                alert(error.status + ': ' + error.statusText);
            }
        });
    },

    filterSetListOnChange: function (filterSetId) {

        if (filterSetId == "chooseFilterSet") filterSetId = "";

        window.location.href = this.constructUrlWithFilterSetId(filterSetId);
    },

    showFilterSetListDialog: function () {
        $("#filterSetsModal").modal("show");
    },

    filterSetListOnItemClick: function (filterSetId) {

        $("#filterSetsModal").modal("hide");

        window.location.href = this.constructUrlWithFilterSetId(filterSetId);
    },

    constructUrlWithFilterSetId: function (filterSetId) {

        let urlparts = window.location.href.split('?');

        if (urlparts.length == 2) {

            let parameters = urlparts[1].split('&');

            let includesFilterSetId = false;

            for (let i = 0; i < parameters.length; i++) {
                if (parameters[i].includes("filterSetId")) {
                    parameters[i] = "filterSetId=" + filterSetId;
                    includesFilterSetId = true;
                }
            }

            return urlparts[0] + "?" + parameters.join('&') + (!includesFilterSetId ? "&filterSetId=" + filterSetId : "");
        }

        return urlparts[0] + "?filterSetId=" + filterSetId;
    },

    clearFilterSet: function () {
        window.location.href = window.location.href.split('?')[0];
    },

    getFastSearchIdNamePairs: function (controllerName, listName) {

        let select = document.getElementsByName(listName)[0];

        let selectElementOptions = select.childNodes;

        for (let i = 0; i < selectElementOptions.length; i++) {
            if (selectElementOptions[i].value === '') {
                select.removeChild(selectElementOptions[i]);
            }
        }

        let urlParts = window.location.href.split('/');
        let host = urlParts[0] + "//" + urlParts[2];

        if (controllerName.includes("TaskQueues/Details"))
            controllerName = host + "/TaskQueues";

        if (controllerName.includes("Queues/Details") && !controllerName.includes("Task"))
            controllerName = host + "/Queues";

        if (select.childElementCount == 0) {
            $.ajax({
                url: controllerName + "/GetFastSearchIdNamePairs",
                contentType: "application/x-www-form-urlencoded",
                method: "GET",
                data: {
                    listName: listName
                },
                success: function (response) {

                    let defaultOptionElement = document.createElement('option');
                    defaultOptionElement.value = 'default';
                    select.appendChild(defaultOptionElement);

                    for (let i = 0; i < response.length; i++) {
                        let optionElement = document.createElement('option');
                        optionElement.value = response[i].Key;
                        optionElement.innerHTML = response[i].Value;
                        select.appendChild(optionElement);
                    }
                }
            });
        }
    }
}