﻿   //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\ Описание типов //////////////////////////////////

    // Тип для описания состояния схемы
    function StageState(connectors, targets, stage) {
        this.connectors = connectors;
        this.targets = targets;
        this.stage = stage;
    }

//// Блок для задачи
//function TaskBlock(type) {
//    this.blockType = type;
//}

//// Блок для условного оператора
//function ConditionBlock(type) {
//    this.blockType = type;
//}

    // Тип для описания блока
    function Target(type, block, taskId, blockId, nextBlockId, text, ports) {
        this.blockType = type;
        this.block = block;
        this.taskId = taskId;
        this.blockId = blockId;
        this.nextBlockId = nextBlockId;
        this.text = text;
        this.ports = ports;
    }

    // Тип для описания соединения
    function Connector(line, from, to, inPort) {
        this.line = line;
        this.sourceId = from;
        this.destId = to;
        this.inPort = inPort;

        this.line.on('click', setLineActive);
        this.line.on('mouseover', function () {
            this.attrs.stroke = "#5A99FA";
            this.attrs.strokeWidth = 3;
            console.log("arrow mouseover");
            layer.batchDraw();
        });
        this.line.on('mouseout', function () {
            this.attrs.stroke = "#636A8A";
            this.attrs.strokeWidth = 2;
            console.log("arrow mouseout");
            layer.batchDraw();
        });
    }

    /////////////////////////////////////////////////////////////////////////////////////

    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\ Сохранение/Восстановление схемы //////////////////////////////

    function saveScheme() {

        let stageState = new StageState(connectors, targets, stage);

        let schemeJSON = serializeStateObject(stageState);

        return schemeJSON

        // TODO: Добавить отправку в контроллер
    }

    function restoreScheme(schemeJSON, executionMode = false) {

        // Десериализуем объект схемы
        let scheme = schemeJSON;

        // Очищаем слой
        layer.destroyChildren();

        // очищаем массивы
        connectors = [];
        targets = [];

        // восстанавливаем блоки
        scheme.targets.forEach(target => {

            let restoredBlock = Konva.Node.create(target.block);
            //let restoredText = Konva.Node.create(target.text);

            let restoredInPortLeft;
            let restoredInPortTop;
            let restoredInPortBottom;
            let restoredOutPort;


            if (target.blockId === "Start") {
                restoredOutPort = Konva.Node.create(target.ports.outPort);
                layer.add(restoredBlock, restoredOutPort);

                // Восстанавливаем кастомную функцию рисования иконки блока
                let customIcon = restoredBlock.getChildren(node => node.attrs.id === 'startCustomShape');
                setStartBlockSceneFunc(customIcon);
            }
            else {
                // Восстанавливаем кастомную функцию рисования иконки блока
                let customIcon = restoredBlock.getChildren(node => node.attrs.id === 'taskCustomShape'); 
                setTaskBlockSceneFunc(customIcon);

                restoredInPortLeft = Konva.Node.create(target.ports.inPortLeft);
                restoredInPortTop = Konva.Node.create(target.ports.inPortTop);
                restoredInPortBottom = Konva.Node.create(target.ports.inPortBottom);
                restoredOutPort = Konva.Node.create(target.ports.outPort);

                layer.add(restoredBlock,
                            restoredInPortLeft,
                            restoredInPortTop,
                            restoredInPortBottom,
                            restoredOutPort);
            }

            let ports = {
                inPortLeft: restoredInPortLeft,
                inPortTop: restoredInPortTop,
                inPortBottom: restoredInPortBottom,
                outPort: restoredOutPort
            };

            let newTarget = new Target(target.blockType, restoredBlock, target.taskId, target.blockId, target.nextBlockId, null, ports);

            targets.push(newTarget);

            //layer.add(restoredBlock, restoredText);
        });

        // Восстанавливаем соединения между блоками
        scheme.connectors.forEach(connector => {
            let newLine = Konva.Node.create(connector.line);

            // Добавляем линию на слой
            layer.add(newLine);

            connectors.push(new Connector(newLine, connector.sourceId, connector.destId, connector.inPort));
        });

        // перерисовываем слой
        layer.batchDraw();

        // Восстанавливаем подписки
        if (!executionMode)
            restoreSubscribes();
        //else {
        //    targets.forEach(target => {
        //        target.block.setAttrs({
        //            draggable: false
        //        });
        //    });
        //}
    }

    function restoreSubscribes() {

        stage.on('dblclick', stageDblClick);
        stage.on('mousemove', function (e) {
            changeLineCoords(e);
        });
        
        stage.on('mouseup', function () {

            if (curConnection != undefined) {

                curConnection.line.destroy();
                curConnection = undefined;
                layer.batchDraw();
            }
        });

        targets.forEach(target => {

            subscribeBlockEvent(target.block, target);

            if (target.blockId == "Start")
                subscribePortEvents([target.ports.outPort]);
            else
                subscribePortEvents([target.ports.inPortTop,
                                        target.ports.inPortLeft,
                                        target.ports.inPortBottom,
                                        target.ports.outPort]);

            //subscribeTextEvents(target.text); 
        });

        layerEvents(stage, layer, targets);
    }


    let stage = new Konva.Stage({
        container: "wfdesigner_stage",
        width: document.getElementById("wfdesigner_stage").clientWidth,
        height: document.getElementById("wfdesigner_stage").clientHeight,
        id: 'mainStage',
        draggable: true
    });


    // Объект с информацией о текущих точках соенинения
    var linePoints = { fromId: undefined, toId: undefined };

// Объект отвечающий за выбранный инструмент
var selectedTool = undefined;

    // слой рабочей области
    let layer = new Konva.Layer();
    stage.add(layer);

    // Активный элемент схемы
    var activeSchemeItem;

    // Все блоки схемы
    var targets = [];

    // Массив соединений между элементами
    var connectors = [];

    // храним соединение до завершения DnD
    var curConnection;

    
    var saveBtn = document.getElementById('saveProperty');

// сохранить выбранную задачу 
saveBtn.onclick = function () {

    closeModalBySave();
    saveTextProperty();
}

    // Сохраняем выбор задачи для блока
    function saveTextProperty() {

        let sel = document.getElementById("taskSelect");
        let taskName = sel.options[sel.selectedIndex].text;
        let taskId = sel.options[sel.selectedIndex].value;

        // Находим нужный объект в массиве
        let curTarget = targets.find(target => target.block == activeSchemeItem);

        if (curTarget != undefined) {

            curTarget.taskId = taskId;

            let text = curTarget.block.getChildren(function (node) {
                return node.getClassName() === 'Text';
            });

            // Изменяем задачу
            text.setAttrs({
                text: taskName
            });
        }
        else {
            console.log('can not find target object');
        }

        layer.batchDraw();
    }


function addText(text, x, y, width) {

    let blockText = new Konva.Text({
        text: text,
        x: x,
        y: y,
        width: width,
        draggable: false,
        fill: '#050F3A',
        fontSize: 18,
        padding: 5,
        align: 'center'
    });

    subscribeTextEvents(blockText);

    return blockText;
}

 

function subscribeTextEvents(text) {

    //text.on("dragmove", dragmoveEvent);
    text.on("click", function () {

        let target = targets.find(item => item.text == text);

        if (target === undefined)
            console.log('target is undefined');
        else
            clickEvent(target.block);
    });

    text.on("dblclick", function (evt) {
        evt.cancelBubble = true;

        let target = targets.find(item => item.text == text);
        
        if (target.block.attrs.id != "start")
            showModal();
    });

    text.on('mouseup', function (evt) {
        evt.cancelBubble = true;
        let target = targets.find(item => item.text == text);

        if (target.block.attrs.id != "start")
            finishConnection(target.ports.inPort, target);
    });

    text.on('mouseover', function (evt) {
        let target = targets.find(item => item.text == text);
        showPorts(target.block, target);
        evt.cancelBubble = true;
    });

    text.on('mouseout', function (evt) {
        evt.cancelBubble = true;
        let target = targets.find(item => item.text == text);
        hidePorts(target.block, target);
    });
}

    function getPortsCoords(target, portSize=10) {

        let inPortLeftX = target.block.attrs.x - portSize;
        let inPortLeftY = target.block.attrs.y + target.block.attrs.height / 2 - portSize / 2;

        let inPortTopX = target.block.attrs.x + target.block.attrs.width/2 - portSize/2;
        let inPortTopY = target.block.attrs.y - portSize;

        let inPortBottomX = target.block.attrs.x + target.block.attrs.width/2 - portSize/2;
        let inPortBottomY = target.block.attrs.y + target.block.attrs.height;


        let outPortX = target.block.attrs.x + target.block.attrs.width;
        let outPortY = target.block.attrs.y + target.block.attrs.height / 2 - portSize / 2;

        return {
            inPortLeftX: inPortLeftX,
            inPortLeftY: inPortLeftY,

            inPortTopX: inPortTopX,
            inPortTopY: inPortTopY,

            inPortBottomX: inPortBottomX,
            inPortBottomY: inPortBottomY,

            outPortX: outPortX,
            outPortY: outPortY
        }
    }

function addStartBlock(x,y) {

    let startBlock = createStartBlockGroup(x, y); 

    // Создаем объект блока
    let target = new Target('system', startBlock, 'no task', startBlock.attrs.id, null,null);
    targets.push(target);

    // Добавляем выходной порт
    startBlockPort = addStartBlockPort(target);
    target.ports = { outPort: startBlockPort };
     
    // Подписываемся на события старт-блока
    subscribeBlockEvent(startBlock, target);

    // Подписываемся на события порта старт-блока
    subscribePortEvents([startBlockPort]);

    layer.add(startBlock, startBlockPort);
}


function addStartBlockPort(target, portSize = 10) {

    let outPort = CreateRectangle('outPort' + target.blockId);

    let outPortX = target.block.attrs.x + target.block.attrs.width;
    let outPortY = target.block.attrs.y + target.block.attrs.height / 2 - portSize / 2;

    outPort.setAttrs({
        x: outPortX,
        y: outPortY,
        width: portSize,
        height: portSize,
        fill: '#5A99FA',
        cornerRadius: 2,
        draggable: false,
        id: 'outPort',
        visible: false
    });

    return outPort;
}

    function addBlockPorts(target) {

        let portSize = 10;
        let portsCoords = getPortsCoords(target, portSize);
         
        let inPortLeft = CreateRectangle('inPortLeft' + target.blockId);

        inPortLeft.setAttrs({
            x: portsCoords.inPortLeftX,
            y: portsCoords.inPortLeftY,
            width: portSize,
            height: portSize,
            fill: '#5A99FA',
            cornerRadius: 0,
            draggable: false,
            id: 'inPortLeft',
            visible: false
        });

        let inPortTop = CreateRectangle('inPortTop' + target.blockId);

        inPortTop.setAttrs({
            x: portsCoords.inPortTopX,
            y: portsCoords.inPortTopY,
            width: portSize,
            height: portSize,
            fill: '#5A99FA',
            cornerRadius: 0,
            draggable: false,
            id: 'inPortTop',
            visible: false
        });

        let inPortBottom = CreateRectangle('inPortBottom' + target.blockId);

        inPortBottom.setAttrs({
            x: portsCoords.inPortBottomX,
            y: portsCoords.inPortBottomY,
            width: portSize,
            height: portSize,
            fill: '#5A99FA',
            cornerRadius: 0,
            draggable: false,
            id: 'inPortBottom',
            visible: false
        });

        let outPort = CreateRectangle('outPort' + target.blockId);

        outPort.setAttrs({
            x: portsCoords.outPortX,
            y: portsCoords.outPortY,
            width: portSize,
            height: portSize,
            fill: '#5A99FA',
            cornerRadius: 0,
            draggable: false,
            id: 'outPort',
            visible: false
        });

        return {
            inPortLeft: inPortLeft,
            inPortTop: inPortTop,
            inPortBottom: inPortBottom,
            outPort: outPort
        }
    }

    function subscribePortEvents(ports) {
        ports.forEach(port => {
            
            port.on('mouseup', function (event) {
                event.cancelBubble = true; 
                finishConnection(this);
            });

            port.on('mousedown', function (event) {
                startConnection(this);
                event.cancelBubble = true;
            }); 

            port.on('mouseover', function (event) {
                this.show();
                layer.draw();
                event.cancelBubble = true;
            });

            port.on('mouseout', function (event) {
                this.hide();
                layer.draw();
                event.cancelBubble = true;
            });
        });
    }

stage.on('mousemove', function (e) {
    changeLineCoords(e);
});



stage.on('mouseup', function () {

    if (curConnection != undefined) {

        curConnection.line.destroy();
        curConnection = undefined;
        layer.batchDraw();
    }
});

 
    function startConnection(port) {

        if (port.attrs.id === "outPort") {
            let resCoords = port.position();
            let x = resCoords.x + port.attrs.width / 2;
            let y = resCoords.y + port.attrs.height / 2;

            let target = targets.find(target => target.ports.outPort == port);

            curConnection = new Connector(createLine([x, y], 'line' + connectors.length + 1), target.block.attrs.id, undefined, null);
            // линияя всегда на заднем плане 
            layer.add(curConnection.line);

            curConnection.line.zIndex(0);
        }
        else {
            console.log('can\'t start connection from inPort! Select outPort, please.');
        }
    }

function CheckFinishConnectionConditions(port) {

    if (port != undefined && curConnection != undefined
                            && (port.attrs.id === "inPortLeft"
                                || port.attrs.id === "inPortTop"
                                || port.attrs.id === "inPortBottom"))
        return true;
    else
        false;

}

function finishConnection(port, target = undefined) {

    if (CheckFinishConnectionConditions(port)) {

        // Находим соединяемый блок
        if (target === undefined)
            target = targets.find(target => target.ports.inPortLeft == port
                                            || target.ports.inPortTop == port
                                            || target.ports.inPortBottom == port);

        if (curConnection.line != undefined && curConnection.sourceId != target.block.attrs.id) {

            let lineCoords = curConnection.line.points();

            let endX = port.attrs.x + port.attrs.width / 2;
            let endY = port.attrs.y + port.attrs.height / 2;

            // lineCoords[0] и lineCoords[1] - x, y начала линии
            let newCoords = [lineCoords[0], lineCoords[1], endX, endY];

            curConnection.destId = target.block.attrs.id;
            curConnection.inPort = port.attrs.id;

                // Для sourceId устанавливаем destId в качестве nextBlockId
            setNextBlock(curConnection.sourceId, curConnection.destId);

            curConnection.line.points(newCoords);
            layer.batchDraw();

            connectors.push(curConnection);
        }
        else {
            if (curConnection.line != undefined)
                curConnection.line.destroy();
        }
        
        curConnection = undefined;
    }
}

    function changeLineCoords(e) {

        if (curConnection != undefined) {

            if (curConnection.line != undefined) {

                let cursor = stage.getPointerPosition(); //{ x: e.x, y: e.y };
                let curCord = curConnection.line.points();
                let stagePos = stage.position();
                // делаем cursor.x - 1, т.к. без этого событие mouseup срабатывает на линии, а не порте
                let newCords = [curCord[0], curCord[1], cursor.x - stagePos.x - 1, cursor.y - stagePos.y-1];

                curConnection.line.points(newCords);
                layer.batchDraw();
            }
            else
                curConnection = undefined;
        }
    }

function showPorts(block, target = undefined) {

    if (target ===  undefined)
        target = targets.find(target => target.block == block);

    if (target != undefined) {

        if (target.ports.inPortLeft != undefined)
            target.ports.inPortLeft.show();

        if (target.ports.inPortTop != undefined)
            target.ports.inPortTop.show();

        if (target.ports.inPortBottom != undefined)
            target.ports.inPortBottom.show();


        if (target.ports.outPort != undefined)
            target.ports.outPort.show();

        layer.draw();
    }
}

function hidePorts(block) {
    let target = targets.find(target => target.block == block);

    if (target != undefined) {

        if (target.ports.inPortLeft != undefined)
            target.ports.inPortLeft.hide();

        if (target.ports.inPortTop != undefined)
            target.ports.inPortTop.hide();

        if (target.ports.inPortBottom != undefined)
            target.ports.inPortBottom.hide();


        if (target.ports.outPort != undefined)
            target.ports.outPort.hide();
         
        layer.draw();
    }
}


 

    // Добавляем прямоугольник в рабочую область
function addBlockToScheme(newItem) {

    if (newItem === undefined)
        newItem = createBlockGroup('rect' + targets.length.toString() + 1)
        //newItem = CreateRectangle('rect' + targets.length.toString() + 1);

    if (targets.find(item => item.blockId == "Start") == undefined) 
        addStartBlock(newItem.attrs.x - 200, newItem.attrs.y); 

    // Добавляем новый блок в массив прямоугольников
    let target = new Target('task', newItem, 'no task', newItem.attrs.id, null,null);
    targets.push(target);

    //let textX = target.block.attrs.x;
    //let textY = target.block.attrs.y + target.block.attrs.height / 4;
    //let textWidth = target.block.attrs.width;

    //// Добавляем текст с id задачи
    //let text = addText(target.taskId, textX, textY, textWidth);
    //target.text = text;

    // Создаем порты для блока
    let ports = addBlockPorts(target);
    target.ports = ports;

    // Цепляем обработчики
    subscribeBlockEvent(newItem, target);

    // Подписка на события портов
    subscribePortEvents([target.ports.inPortTop, target.ports.inPortLeft, target.ports.inPortBottom, target.ports.outPort]); 

    // Добавляем объекты на слой
    layer.add(newItem, ports.inPortTop, ports.inPortLeft, ports.inPortBottom, ports.outPort);
    layer.batchDraw();

}


function subscribeBlockEvent(newItem, target)
{
    newItem.dragDistance(5);

    newItem.on("dragmove", dragmoveEvent);

    newItem.on("click", function () {
        clickEvent(newItem);
    });

    newItem.on("dblclick", function (event) {  
        event.cancelBubble = true;

        if (newItem.attrs.id != "Start")
            showModal(); 
    });

    newItem.on('mouseup', function (evt) {
        evt.cancelBubble = true;

        if (newItem.attrs.id != "Start")
            finishConnection(target.ports.inPortLeft, target); 
    });

    newItem.on('mouseover', function (event) {
        showPorts(newItem);
        event.cancelBubble = true;
    });

    newItem.on('mouseout', function (event) {
        hidePorts(newItem);
        event.cancelBubble = true;
    });
}

function createBlockGroup(name, x, y) {

    x = (x == undefined) ? 140 : x;
    y = (y == undefined) ? 30 : y;
     
    let group = new Konva.Group({
        x: x,
        y: y,
        width: 231,
        height: 58,
        draggable: true,
        id: name
    });
     
    group.add(new Konva.Rect({
        id: name,
        width: group.attrs.width,
        height: group.attrs.height,
        fill: '#D9E8FF',
        stroke: '#AFC5E7',
        strokeWidth: 1,
        name: name
    }));


    group.add(new Konva.Text({
        text: 'no Task',
        y: group.attrs.height / 4,
        fontSize: 18,
        fontFamily: 'Roboto',
        fill: '#000',
        width: group.attrs.width,
        padding: 5,
        align: 'center'
    }));

    group.add(new Konva.Rect({
        x: 20,
        y: 20,
        width: 17,
        height: 17,
        stroke: '#5A99FA',
        strokeWidth:1
    }));

    group.add(new Konva.Shape({
        sceneFunc: function (context) {
            context.beginPath();
            context.moveTo(26, 21);
            context.lineTo(34, 29);
            context.lineTo(26, 37);
            context.closePath();

            // Konva specific method
            context.fillStrokeShape(this);
        },
        fill: '#5A99FA',
        strokeWidth: 0,
        id:"taskCustomShape"
    }));
     
    return group;
}



function createStartBlockGroup(x,y) {
     
    let group = new Konva.Group({
        x: x,
        y: y,
        width: 104,
        height: 76,
        draggable: true,
        id: 'Start'
    }); 
      
    group.add(new Konva.Rect({
        id:'Start',
        width: group.attrs.width,
        height: group.attrs.height,
        fill: '#FFFFFF',
        stroke: '#D9DAE2',
        strokeWidth: 1
    }));


    group.add(new Konva.Text({
        text: 'Start',
        y: group.attrs.height / 3.5,
        fontSize: 18,
        fontFamily: 'Roboto',
        fill: '#000',
        width: group.attrs.width,
        padding: 5,
        align: 'center'
    }));

    group.add(new Konva.Rect({
        width: 10,
        height: group.attrs.height,
        fill: 'black'
    })); 
      
    group.add(new Konva.Shape({
        sceneFunc: function (context) {
            context.beginPath();
            context.moveTo(20, 28);
            context.lineTo(30, 35);
            context.lineTo(20, 43);
            context.closePath();

            // Konva specific method
            context.fillStrokeShape(this);
        },
        fill: '#28BB48',
        strokeWidth: 0,
        id: "startCustomShape"
    }));
     
       
    return group;
}

function setStartBlockSceneFunc(block) {

    block.setSceneFunc(function (context) {
        context.beginPath();
        context.moveTo(20, 28);
        context.lineTo(30, 35);
        context.lineTo(20, 43);
        context.closePath();

        // Konva specific method
        context.fillStrokeShape(this);
    });
}

function setTaskBlockSceneFunc(block) {

    block.setSceneFunc(function (context) {
        context.beginPath();
        context.moveTo(26, 21);
        context.lineTo(34, 29);
        context.lineTo(26, 37);
        context.closePath();

        // Konva specific method
        context.fillStrokeShape(this);
    });
}
 

function CreateRectangle(name, x = 140, y = 30, stroke = '#AFC5E7', width = 231, height = 58, fill = '#D9E8FF', draggable = true, cornerRadius = 1) {
     
        let res = new Konva.Rect({
            x: x,
            y: y,
            width: width,
            height: height,
            fill: fill,
            name: name,
            id: name,
            cornerRadius: cornerRadius,
            stroke: stroke,
            strokeWidth:1,
            draggable: draggable
        });

        return res;
    }


    // Создаем соединение между элементами схемы
    function createLine(points, name) {

        let redLine = new Konva.Arrow({
            points: points,
            stroke: '#636A8A',
            strokeWidth: 2,
            lineCap: 'round',
            lineJoin: 'round',
            draggable: false,
            id: name,
            name: name
        });

        return redLine;
    }

function getArrayFromObject(comlexObject) {
    let result = Object.keys(comlexObject).map(i => comlexObject[i]);
     
    return result;
}

    // Вычисление углов создания соединений
    function getConnectorPoints(from, to, inPortId) {

        let outX = from.ports.outPort.attrs.x + from.ports.outPort.attrs.width / 2;
        let outY = from.ports.outPort.attrs.y + from.ports.outPort.attrs.height / 2;

        let testPorts = getArrayFromObject(to.ports);
        let destPort = testPorts.find(item => item.attrs.id === inPortId);

        let inX = destPort.attrs.x + destPort.attrs.width / 2;
        let inY = destPort.attrs.y + destPort.attrs.height / 2;
         
        let source = { x: outX, y: outY };
        let dest = { x: inX, y: inY };

        const dx = dest.x - source.x;
        const dy = dest.y - source.y;
        let angle = Math.atan2(-dy, dx);

        return [
            source.x + Math.cos(angle + Math.PI),
            source.y + Math.sin(angle + Math.PI),
            dest.x + Math.cos(angle),
            dest.y + Math.sin(angle)
        ];
    }


    // Обновляем состояние координаты линий соединения
    function updateObjects() {
         
        //blockTextUpdater();

        portsCoorsUpdater();

        connectorUpdater();

        layer.batchDraw();
    }


    function dragmoveEvent() {
        // update nodes from the new state
        updateObjects();
    }

    var portsCoorsUpdater = function () {
        targets.forEach(target => {

            let portsCoords = getPortsCoords(target);

            if (target.ports.inPortLeft != undefined) {
                target.ports.inPortLeft.setAttrs({
                    x: portsCoords.inPortLeftX,
                    y: portsCoords.inPortLeftY
                });

                target.ports.inPortTop.setAttrs({
                    x: portsCoords.inPortTopX,
                    y: portsCoords.inPortTopY
                });

                target.ports.inPortBottom.setAttrs({
                    x: portsCoords.inPortBottomX,
                    y: portsCoords.inPortBottomY
                });
            } 


            if (target.ports.outPort != undefined) {
                target.ports.outPort.setAttrs({
                    x: portsCoords.outPortX,
                    y: portsCoords.outPortY
                });
            }
        });
    };

    // Обновление координат текста блока
    var blockTextUpdater = function () {

        targets.forEach(target => {
            target.text.setAttrs({
                x: target.block.attrs.x,
                y: target.block.attrs.y + target.block.attrs.height / 4
            });
        }); 
    };

    // Обновление координат линии по координатам блоков
    var connectorUpdater = function () {
        connectors.forEach(connector => {

            let source = layer.findOne('#' + connector.sourceId);
            let dest = layer.findOne('#' + connector.destId);

            let sourceTarget = targets.find(target => target.block == source);
            let destTarget = targets.find(target => target.block == dest);

            let resCoords = getConnectorPoints(sourceTarget, destTarget, connector.inPort);

            connector.line.points(resCoords);
        });
    };

    // Сброс информации о текущих точках соединения
    var resetLinePoints = function () {
        linePoints.fromId = undefined;
        linePoints.toId = undefined;
    }


    function resetStrokeSettings() {

        targets.forEach(target => {

            if (target.blockId === "Start")
                target.block.attrs.stroke = "#D9DAE2";
            else
                target.block.attrs.stroke = "#AFC5E7";

            target.block.attrs.strokeWidth = 1;
        });

        connectors.forEach(connect => {
            connect.line.attrs.stroke = "#636A8A";
            connect.line.attrs.strokeWidth = 2;
        });
    }


    // Выделение блока в качестве активного элемента
    function setBlockActive(curItem) {

        // Меняем активный элемент
        activeSchemeItem = curItem;

        if (curItem.attrs.stroke === "#5A99FA") {

            if (curItem.attrs.id === "Start")
                curItem.attrs.stroke = "#D9DAE2";
            else
                curItem.attrs.stroke = "#AFC5E7"; 

            curItem.attrs.strokeWidth = 1;
        }
        else {
            curItem.attrs.stroke = "#5A99FA";
            curItem.attrs.strokeWidth = 3;
        }
    }

function mouseOverHandler() {
    
    //    this.attrs.stroke = "gray"; 
    //else {
    //    this.attrs.stroke = "red";
    //}
}

    // Выделение линии в качестве активного элемента
    function setLineActive() {

    // "#D9E8FF"  "#636A8A";


        // Меняем активный элемент
        activeSchemeItem = this;

        if (this.attrs.stroke === "red") {
            this.attrs.stroke = "#636A8A";
            this.attrs.strokeWidth = 2;
        }
        else {
            this.attrs.stroke = "red";
            this.attrs.strokeWidth = 3;
        }
    }

    // Устанавливаем в блок адрес следующего блока
    function setNextBlock(sourceId, destId) {

        let sourceTarget = targets.find(target => target.blockId == sourceId);

        sourceTarget.nextBlockId = destId;

    }

    // Событие клика на элемент
function clickEvent(block) {

    // Помечаем выбранный элемент активным
    if (block != undefined) {
        if (activeSchemeItem === block) {
            setBlockActive(block);
        }
        else {
            resetStrokeSettings();
            setBlockActive(block);
        }

        layer.batchDraw();
    }
}


    stage.on('dblclick', stageDblClick);

    function stageDblClick() {

        console.log('stage clicked');
        resetLinePoints();
        resetStrokeSettings();

        activeSchemeItem = undefined;

        layer.batchDraw();

    }

    var container = stage.container();
    container.tabIndex = 1;
    container.focus();

    container.addEventListener('dragover', function (e) {
        e.preventDefault(); // !important
    });

container.addEventListener('drop', function (e) {

    e.preventDefault();

    stage.setPointersPositions(e);

    let cursorCoords = stage.getPointerPosition();
    
    let stagePos = stage.position(); 
    
    if (selectedTool === 'task') {

        let x = cursorCoords.x - stagePos.x - 115;
        let y = cursorCoords.y - stagePos.y - 29;

        let newblock = createBlockGroup('rect' + targets.length.toString() + 1, x, y);
            //CreateRectangle();
          
        //newblock.setAttrs({
        //    x: cursorCoords.x - stagePos.x - 70,
        //    y: cursorCoords.y - stagePos.y - 70
        //});

        addBlockToScheme(newblock);

        // сбрасываем выбор елемента
        selectedTool = undefined;
    }
    //console.log(selectedTool + ' /// ' + corrr.x + '  ' + corrr.y);
});

    // Удаляем элемент с рабочей области
container.addEventListener('keydown', function (e) {
    console.log(e.which + "//////"+ e.button);
    destroySchemeItem(e);
});

//container.addEventListener('mousemove', function (e) {
//    e.preventDefault();
     
//    if (cursorPositionGlob != undefined ) {
        
//        // Текущая позиция курсора
//        let curPos = stage.getPointerPosition();

//        // Вычисляем сдвиг курсора
//        let newPos = {
//            x: curPos.x - cursorPositionGlob.x,
//            y: curPos.y - cursorPositionGlob.y
//        } 

//        // Двигаем сцену
//        stage.setAttrs({
//            x: newPos.x,
//            y: newPos.y
//        }); 

//        layer.batchDraw();
//    }
//});

//// Позиция курсора для вычисления сдвига сцены
//var cursorPositionGlob = undefined;

//container.addEventListener('mousedown', function (e) {
//    e.preventDefault();

//    if (e.button === 1) {
        
//        cursorPositionGlob = stage.getPointerPosition();
//        //console.log("cursor position x: " + curCoor.x + "y: " + curCoor.y); 
//    }
//});

//container.addEventListener('mouseup', function (e) {
//    e.preventDefault();

//    if (cursorPositionGlob != undefined) {
//        cursorPositionGlob = undefined
//    } 
//});

    // Удаление блока
    function deleteBlock(delTarget) {

        // Удяляем все ссылки на блок
        targets.forEach(target => {
            if (target.nextBlockId === delTarget.blockId)
                target.nextBlockId = null;
        });

        let delConnectors = connectors.filter(item => item.sourceId === delTarget.blockId || item.destId === delTarget.blockId);

        delConnectors.forEach(delCon => {
            deleteConnection(delCon);
        });

        // Находим индекс активного элемента в массиве
        let itemIndex = targets.indexOf(delTarget, 0);
        delete targets[itemIndex];
        targets = targets.filter(function (item) { return item; });

        delTarget.ports.inPortLeft.destroy();
        delTarget.ports.inPortTop.destroy();
        delTarget.ports.inPortBottom.destroy();
        delTarget.ports.outPort.destroy();
        //delTarget.text.destroy();
        delTarget.block.destroy();
    }

    // Удаление соединителя
    function deleteConnection(connector) {

        let itemIndex = connectors.indexOf(connector, 0);
        delete connectors[itemIndex];

        connectors = connectors.filter(function (item) { return item.line != null; });
        connector.line.destroy();
    }


    function destroySchemeItem(e) {

        if (e.keyCode === 46) {

            let type = activeSchemeItem.getClassName();

            if (type === 'Group') {

                // Удяляем блок из массива блоков
                let target = targets.find(item => item.block == activeSchemeItem);

                deleteBlock(target);
            }
            else
                if (type === 'Arrow') {

                    // Удяляем блок
                    let connector = connectors.find(item => item.line == activeSchemeItem);
                    if (connector != undefined)
                        deleteConnection(connector);
                    else
                        activeSchemeItem.destroy();
                }

            activeSchemeItem = undefined;
            layer.batchDraw();
        }
    }

function highlightBlock(blockID) {
    let target = targets.find(item => item.blockId === blockID);

    targets.forEach(item => {
        item.block.setAttrs({
            shadowColor: 'green',
            shadowBlur: 0,
            //shadowOffset: { x: 10, y: 10 },
            shadowOpacity: 0
        });
    });

    target.block.setAttrs({
        shadowColor: 'green',
        shadowBlur: 30,
        //shadowOffset: { x: 10, y: 10 },
        shadowOpacity: 0.7
    });

    layer.batchDraw();
}

function finishProcessExecuting() {
    //shadowEnabled: false
    targets.forEach(item => {
        item.block.setAttrs({ 
            shadowColor: 'green',
            shadowBlur: 0,
            shadowOpacity: 0
        });
    });

    layer.batchDraw();
}

function restrictEditing() {

    
}

    updateObjects();

    // Подключаем подсказки по выравниванию блоков
    layerEvents(stage, layer, targets);
