﻿function BlockStageState(blocks) {
    this.Blocks = blocks;
}

function AbstractBlockTarget(id, blockName, coordinateX, coordinateY, connections, ports) {
    this.Id = id;
    this.BlockName = blockName;

    this.CoordinateX = coordinateX;
    this.CoordinateY = coordinateY;

    this.Connections = connections;
    this.Ports = ports;
}

function TaskTarget(abstractBlockTarget, taskId, taskName, taskParameters) {
    this.Type = "task";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.TaskId = taskId;
    this.TaskName = taskName;
    this.TaskParameters = taskParameters;
}

function StartTarget(abstractBlockTarget) {
    this.Type = "start";
    this.Id = abstractBlockTarget.Id;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;
}

function ConditionTarget(abstractBlockTarget, conditionParameters) {
    this.Type = "condition";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.ConditionParameters = conditionParameters;
}

function ParallelExecutionTarget(abstractBlockTarget, innerBlockIds) {
    this.Type = "parallelexecution";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.InnerBlockIds = innerBlockIds;
}

function ConditionParameterTarget(blockId, name, comparisonRelation, comparisonValue) {
    this.BlockId = blockId;
    this.Name = name;
    this.ComparisonRelation = comparisonRelation;
    this.ComparisonValue = comparisonValue;
}

function GetQueueElementTarget(abstractBlockTarget, queueName, queueElementStatusAfterReceiving, reference, priority, comment) {
    this.Type = "getqueueelement";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.QueueName = queueName;
    this.QueueElementStatusAfterReceiving = queueElementStatusAfterReceiving;
    this.Reference = reference;
    this.Priority = priority;
    this.Comment = comment;
}

function PushQueueElementTarget(abstractBlockTarget, queueName, value, priority, postponeDate, deadlineDate, comment, reference, isReferenceUnique) {
    this.Type = "pushqueueelement";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.QueueName = queueName;
    this.Value = value;
    this.Priority = priority;
    this.PostponeDate = postponeDate;
    this.DeadlineDate = deadlineDate;
    this.Comment = comment;
    this.Reference = reference;
    this.IsReferenceUnique = isReferenceUnique;
}

function ConfirmationTarget(abstractBlockTarget, recipientEmail, question, emailSubject, answers) {
    this.Type = "confirmation";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.RecipientEmail = recipientEmail;
    this.Question = question;
    this.EmailSubject = emailSubject;
    this.Answers = answers;
}

function SendEmailTarget(abstractBlockTarget, recipientEmail, emailSubject, emailBody) {
    this.Type = "sendemail";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.RecipientEmail = recipientEmail;
    this.EmailSubject = emailSubject;
    this.EmailBody = emailBody;
}

function HttpRequestTarget(abstractBlockTarget, requestUri, requestMethod, requestTimeout, contentType, requestBody, requestParameters) {
    this.Type = "httprequest";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.RequestUri = requestUri;
    this.RequestMethod = requestMethod;
    this.RequestTimeout = requestTimeout;
    this.ContentType = contentType;
    this.RequestBody = requestBody;

    this.RequestParameters = requestParameters;
}

function HttpRequestParameterTarget(name, source, value, sourceBlockId, sourceBlockParameterName) {
    this.Name = name;
    this.Source = source;
    this.Value = value;
    this.SourceBlockId = sourceBlockId;
    this.SourceBlockParameterName = sourceBlockParameterName;
}

function EmailMonitoringTarget(abstractBlockTarget, mailFolder, markAsRead, filterParameters) {
    this.Type = "emailmonitoring";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.MailFolder = mailFolder;
    this.MarkAsRead = markAsRead;
    this.FilterParameters = filterParameters;
}

function GetKafkaMessageTarget(abstractBlockTarget, bootstrapServers, groupId, topic) {
    this.Type = "getkafkamessage";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.BootstrapServers = bootstrapServers;
    this.GroupId = groupId;
    this.Topic = topic;
}

function GetRabbitMQMessageTarget(abstractBlockTarget, hostName, port, userName, password, virtualHost, queueName) {
    this.Type = "getrabbitmqmessage";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.HostName = hostName;
    this.Port = port;
    this.UserName = userName;
    this.Password = password;
    this.VirtualHost = virtualHost;
    this.QueueName = queueName;
}

function QueueTriggerTarget(abstractBlockTarget, queueName, reference, priority, comment, queueLength, queueUpdateDelay) {
    this.Type = "queuetrigger";
    this.Id = abstractBlockTarget.Id;
    this.BlockName = abstractBlockTarget.BlockName;

    this.CoordinateX = abstractBlockTarget.CoordinateX;
    this.CoordinateY = abstractBlockTarget.CoordinateY;

    this.Connections = abstractBlockTarget.Connections;
    this.Ports = abstractBlockTarget.Ports;

    this.QueueName = queueName;
    this.Reference = reference;
    this.Priority = priority;
    this.Comment = comment;
    this.QueueLength = queueLength;
    this.QueueUpdateDelay = queueUpdateDelay;
}

function ConnectionTarget(sourceBlockId, targetBlockId, sourcePort, targetPort) {
    this.SourceBlockId = sourceBlockId;
    this.TargetBlockId = targetBlockId;
    this.SourcePort = sourcePort;
    this.TargetPort = targetPort;
}

function PortTarget(name, type, position, purpose) {
    this.PortName = name;
    this.Type = type;
    this.Position = position;
    this.Purpose = purpose;
}

function TaskParameterTarget(name, type, source, sourceId, blockParameterName, parameterId) {
    this.Name = name;
    this.Type = type;
    this.Source = source;
    this.SourceId = sourceId;
    this.BlockParameterName = blockParameterName;
    this.ParameterId = parameterId;
}