﻿ParallelExecutionBlock = draw2d.shape.composite.StrongComposite.extend({

    Name: "ParallelExecutionBlock",

    init: function (attr, setter, getter) {
        this._super($.extend({ resizeable: false, bgColor: "#F4F4F4", color: "#C0C0C0", stroke: 0.5, radius: 3 }, attr), setter, getter);

        this.setWidth(300);
        this.setHeight(100);

        if (attr && ("id" in attr)) {
            this.setId(attr.id);
        }

        this.setUserData({});

        this.installEditPolicy(new draw2d.policy.figure.AntSelectionFeedbackPolicy());

        this.stickFigures = false;
        this.innerElementPolicy = new ParallelExecutionBlockInnerElementDragDropEditPolicy(this);
        this.innerBlockHeightStep = 80;

        this.on("removed", function (emitter) {

            let removedFigures = emitter.assignedFigures.clone();

            removedFigures.each(function (i, figure) {
                emitter.unassignFigure(figure);
            });
        });
    },

    onDragEnd: function (x, y, shiftKey, ctrlKey) {
        var _this = this;

        if (this.command !== null && this.canvas !== null) {
            this.command.setPosition(this.x, this.y);
            this.canvas.getCommandStack().execute(this.command);
            this.command = null;
        }

        this.isInDragDrop = false;
        this.panningDelegate = null;

        // notify all installed policies
        //
        if (this.canvas !== null) {
            this.editPolicy.each(function (i, e) {
                if (e instanceof draw2d.policy.figure.DragDropEditPolicy) {
                    e.onDragEnd(_this.canvas, _this, x, y, shiftKey, ctrlKey);
                }
            });
        }

        this.fireEvent("move", { figure: this, dx: 0, dy: 0 });
        this.fireEvent("change:x", { figure: this, dx: 0 });
        this.fireEvent("change:y", { figure: this, dy: 0 });

        // fire an event
        // @since 5.3.3
        this.fireEvent("dragend", { x: x, y: y, shiftKey: shiftKey, ctrlKey: ctrlKey });
    },

    setDimension: function (w, h) {
        this._super(w, h);
    },

    setPosition: function (x, y) {
        var oldX = this.x;
        var oldY = this.y;

        this._super(x, y);

        var dx = this.x - oldX;
        var dy = this.y - oldY;

        if (dx === 0 && dy === 0) {
            return this;
        }

        if (this.stickFigures === false) {
            this.assignedFigures.each(function (i, figure) {
                figure.translate(dx, dy);
            });
        }

        return this;
    },

    assignFigure: function (figure) {
        if (!this.assignedFigures.contains(figure) && figure !== this) {
            this.stickFigures = true;

            let prevAppliedComposite = figure.getComposite();

            if (prevAppliedComposite)
                prevAppliedComposite.unassignFigure(figure);

            this.assignedFigures.add(figure);
            figure.setComposite(this);
            figure.installEditPolicy(this.innerElementPolicy);

            figure.on("removed", function (emitter) {
                emitter.composite.unassignFigure(emitter);
            });

            figure.toFront(this);

            this.centerFigure(figure);
            this.setHeight(this.getHeight() + this.innerBlockHeightStep);

            let ports = figure.getPorts();

            if (ports) {
                ports.each(function (i, port) {
                    port.setVisible(false);
                })
            }

            this.stickFigures = false;
        }

        this.recountInnerBlockPositions();

        return this;
    },

    unassignFigure: function (figure) {
        if (this.assignedFigures.contains(figure)) {
            this.stickFigures = true;
            figure.setComposite(null);
            figure.uninstallEditPolicy(this.innerElementPolicy);
            figure.off("removed");
            this.assignedFigures.remove(figure);

            let ports = figure.getPorts();

            if (ports) {
                ports.each(function (i, port) {
                    port.setVisible(true);
                })
            }

            if (this.canvas) {
                this.recountInnerBlockPositions();
                this.setHeight(this.getHeight() - this.innerBlockHeightStep);
            }

            this.stickFigures = false;
        }

        return this;
    },

    onCatch: function (droppedFigure, x, y, shiftKey, ctrlKey) {
        if (droppedFigure.userData && droppedFigure.userData.blockType === "task" && !this.figureHasConnections(droppedFigure))
            this.getCanvas().getCommandStack().execute(new draw2d.command.CommandAssignFigure(droppedFigure, this));
    },

    figureHasConnections: function (figure) {
        let ports = figure.getPorts();
        let connections = ports.clone().map(function (e) {
            return e.getConnections();
        }).asArray().flatMap(a => a.data);

        return connections.length > 0;
    },

    centerFigure: function (figure) {
        let boundingBox = this.getBoundingBox();

        let step = this.innerBlockHeightStep;
        let assignedFiguresCount = this.getAssignedFigures().getSize() - 1;

        let centerX = boundingBox.x + (boundingBox.getWidth() - figure.width) / 2;
        let centerY = boundingBox.y + (step - figure.height) / 2 + step * assignedFiguresCount;

        figure.setPosition(centerX, centerY);
    },

    recountInnerBlockPositions: function () {
        let step = this.innerBlockHeightStep;
        let boundingBox = this.getBoundingBox();

        this.assignedFigures.each(function (i, af) {
            let centerX = boundingBox.x + (boundingBox.getWidth() - af.width) / 2;
            let centerY = boundingBox.y + (step - af.height) / 2 + step * i;

            af.setPosition(centerX, centerY);
            af.repaint();
        });
    }
});