﻿example.View = draw2d.Canvas.extend({

    init: function (id) {
        this._super(id);

        this.setScrollArea("#" + id);

        let ourDiv = document.getElementById(id);

        let canvasView = this;

        ourDiv.addEventListener("dragover", function (event) {
            event.preventDefault();
            
        });

        ourDiv.addEventListener('drop', (e) => {
            e.preventDefault();

            let blockType = e.dataTransfer.getData("blockType");

            canvasView.addBlockByType(blockType, e.offsetX, e.offsetY);
        });

        ourDiv.addEventListener("click", (e) => {
            document.activeElement.blur();
        });
    },

    addBlockByType: function (blockType, offsetX, offsetY) {
        let canvasView = this;

        $.ajax({
            url: "/BPM/GetBlockByType",
            contentType: "application/x-www-form-urlencoded",
            method: "GET",
            async: false,
            data: {
                blockType: blockType
            },
            success: function (response) {
                let figure = null;

                if (blockType === "start")
                    figure = new StartBlock();
                else if (blockType === "task")
                    figure = new TaskBlock();
                else if (blockType === "condition")
                    figure = new ConditionBlock();
                else if (blockType === "getqueueelement")
                    figure = new GetQueueElementBlock();
                else if (blockType === "pushqueueelement")
                    figure = new PushQueueElementBlock();
                else if (blockType === "parallelexecution")
                    figure = new ParallelExecutionBlock();
                else if (blockType === "confirmation")
                    figure = new ConfirmationBlock();
                else if (blockType === "sendemail")
                    figure = new SendEmailBlock();
                else if (blockType === "httprequest")
                    figure = new HttpRequestBlock();
                else if (blockType === "emailmonitoring")
                    figure = new EmailMonitoringBlock();
                else if (blockType === "getkafkamessage")
                    figure = new GetKafkaMessageBlock();
                else if (blockType === "getrabbitmqmessage")
                    figure = new GetRabbitMQMessageBlock();
                else if (blockType === "queuetrigger")
                    figure = new QueueTriggerBlock();

                figure.id = response.id;
                figure.setUserData({ blockType: response.type, blockName: response.blockName });

                let portPositions = ["Bottom", "Left", "Top", "Right"];

                for (let pos of portPositions) {
                    let portsByPosition = response.ports.filter(f => f.position === pos);
                    portsByPosition = portsByPosition.map(function (el) {
                        return {
                            Position: el.position,
                            Type: el.type,
                            PortName: el.portName,
                            Color: el.color
                        }
                    });
                    createPortsByPosition(figure, portsByPosition, pos);
                }

                let command = new draw2d.command.CommandAdd(canvasView, figure, offsetX, offsetY);
                canvasView.getCommandStack().execute(command);
            }
        });
    },

    setCanvasSize: function (width, height) {

        this.initialWidth = width;
        this.initialHeight = height;

        this.regionDragDropConstraint.constRect.w = width;
        this.regionDragDropConstraint.constRect.h = height;

        this.paper.setSize(this.initialWidth, this.initialHeight);
        this.setZoom(this.zoomFactor, false);
    }
});