﻿function getTaskNameLabel(figure) {
    return getLabelById(figure, "taskName");
}

function getLabelById(figure, labelId) {
    let children = figure.getChildren();
    let text = children.find(item =>
        (item.userData && item.userData.id != undefined)
        && (item.userData.id === labelId)
    );

    return text;
}

function setBlockNameLabel(figure, labelId, value) {
    let conditionNameLabel = getLabelById(figure, labelId);

    if (value.length > 26)
        value = value.substring(0, 25) + "...";

    conditionNameLabel.setText(value);
}

function getPreviousBlockParameters() {
    let sentBlockId = null;
    let parentBlock = selectedBlock.composite;

    if (parentBlock)
        sentBlockId = parentBlock.getId();
    else
        sentBlockId = selectedBlock.getId();

    let scheme = getStageJSON(canvas);

    let blockParameters = {};

    $.ajax({
        url: "/BPM/GetPreviousParametersForBlock",
        contentType: "application/x-www-form-urlencoded",
        method: "POST",
        async: false,
        data: {
            schemeJSON: scheme,
            blockId: sentBlockId
        },
        success: function (response) {
            blockParameters = Object.fromEntries(
                Object.entries(response).filter(([key, value]) => value && value.length > 0));
        }
    });

    return blockParameters;
}

function formPreviousBlockParametersSelectList(previousBlockParameters) {
    let previousBlockParameterControl = document.createElement("select");
    previousBlockParameterControl.className = "parametersSelect";

    for (let blockId of Object.keys(previousBlockParameters)) {
        if (previousBlockParameters[blockId]) {
            let paramOptGroup = document.createElement("optgroup");

            let currentBlock = canvas.view.getFigure(blockId);

            paramOptGroup.dataset.blockId = blockId;
            paramOptGroup.label = currentBlock.userData.blockName;

            for (let blockParam of previousBlockParameters[blockId]) {
                let paramOption = document.createElement("option");
                paramOption.value = blockId + "_" + blockParam;
                paramOption.text = paramOptGroup.label + "." + blockParam;
                paramOption.dataset.name = blockParam;

                paramOptGroup.appendChild(paramOption);
            }

            previousBlockParameterControl.appendChild(paramOptGroup);
        }
    }

    return previousBlockParameterControl;
}

//Сериализация данных draw2d в json для формата ProcessScheme
function serializeDraw2dStageToCustomJSON(stage) {

    let targets = [];
    let connectors = [];

    stage.getLines().each(function (i, element) {
        connectors.push(element);
    });

    stage.getFigures().each(function (i, figure) {
        let figureItem = figure;

        targets.push(getTarget(figureItem, connectors));
    });


    return targets;
}

function getTarget(figure, connectors) {
    let blockType = figure.userData.blockType;

    if (blockType === "start")
        return getStartBlockObject(figure, connectors);
    else if (blockType === "condition") {
        return getConditionBlockObject(figure, connectors);
    }
    else if (blockType === "task") {
        return getTaskBlockObject(figure, connectors);
    }
    else if (blockType === "getqueueelement") {
        return getGetQueueElementBlockObject(figure, connectors);
    }
    else if (blockType === "pushqueueelement") {
        return getPushQueueElementBlockObject(figure, connectors);
    }
    else if (blockType === "parallelexecution") {
        return getParallelExecutionBlockObject(figure, connectors);
    }
    else if (blockType === "confirmation") {
        return getConfirmationBlockObject(figure, connectors);
    }
    else if (blockType === "sendemail") {
        return getSendEmailBlockObject(figure, connectors);
    }
    else if (blockType == "httprequest") {
        return getHttpRequestBlockObject(figure, connectors);
    }
    else if (blockType === "emailmonitoring") {
        return getEmailMonitoringBlockObject(figure, connectors);
    }
    else if (blockType === "getkafkamessage") {
        return getGetKafkaMessageBlockObject(figure, connectors);
    }
    else if (blockType === "getrabbitmqmessage") {
        return getGetRabbitMQMessageBlockObject(figure, connectors);
    }
    else if (blockType === "queuetrigger") {
        return getQueueTriggerBlockObject(figure, connectors);
    }
}

function getAbstractBlockObject(figure, connectors) {
    let blockId = figure.id;
    let blockName = figure.userData.blockName;

    let outputConnectors = connectors.filter(item => item.getSource().getParent().id === figure.id);
    let connectionObjects = [];

    if (outputConnectors)
        connectionObjects = outputConnectors.map(getConnectionObject);

    let ports = figure.getPorts();
    let portObjects = ports.data.map(getPortObject);

    return new AbstractBlockTarget(blockId, blockName, figure.x, figure.y, connectionObjects, portObjects);
}

function getStartBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);

    return new StartTarget(abstractBlock);
}

function getTaskBlockObject(figure, connectors) {
    let taskId = figure.userData.taskId;
    let taskName = getTaskNameLabel(figure).getText();
    let taskParameters = figure.userData.parameters;

    let abstractBlock = getAbstractBlockObject(figure, connectors);

    return new TaskTarget(abstractBlock, taskId, taskName, taskParameters);
}

function getConditionBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);
    let conditionParameters = figure.userData.conditionParameters;

    return new ConditionTarget(abstractBlock, conditionParameters);
}

function getGetQueueElementBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);
    let queueName = figure.userData.queueName;
    let queueElementStatusAfterReceiving = figure.userData.queueElementStatusAfterReceiving;
    let reference = figure.userData.reference;
    let priority = figure.userData.priority;
    let comment = figure.userData.comment;

    return new GetQueueElementTarget(abstractBlock, queueName, queueElementStatusAfterReceiving, reference, priority, comment);
}

function getPushQueueElementBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);
    let queueName = figure.userData.queueName;
    let value = figure.userData.value;
    let priority = figure.userData.priority;
    let postponeDate = figure.userData.postponeDate;
    let deadlineDate = figure.userData.deadlineDate;
    let comment = figure.userData.comment;
    let reference = figure.userData.reference;
    let isReferenceUnique = figure.userData.isReferenceUnique;

    return new PushQueueElementTarget(abstractBlock, queueName, value, priority, postponeDate, deadlineDate, comment, reference, isReferenceUnique);
}

function getParallelExecutionBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);

    let assignedFigures = figure.getAssignedFigures().asArray();
    let innerBlockIds = assignedFigures.map(m => m.id);

    return new ParallelExecutionTarget(abstractBlock, innerBlockIds);
}

function getConfirmationBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);

    let recipientEmail = figure.userData.recipientEmail;
    let question = ParametersInTextHelper.convertCKEditorTagsToCommonFormat(figure.userData.question);
    let emailSubject = figure.userData.emailSubject;
    let answers = figure.userData.answers;

    return new ConfirmationTarget(abstractBlock, recipientEmail, question, emailSubject, answers);
}

function getSendEmailBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);

    let recipientEmail = figure.userData.recipientEmail;
    let emailSubject = figure.userData.emailSubject;
    let emailBody = ParametersInTextHelper.convertCKEditorTagsToCommonFormat(figure.userData.emailBody);

    return new SendEmailTarget(abstractBlock, recipientEmail, emailSubject, emailBody);
}

function getHttpRequestBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);

    let requestUri = figure.userData.requestUri;
    let requestMethod = figure.userData.requestMethod;
    let requestTimeout = figure.userData.requestTimeout;
    let contentType = figure.userData.contentType;
    let requestBody = figure.userData.requestBody;

    let requestParameters = figure.userData.requestParameters;

    return new HttpRequestTarget(abstractBlock, requestUri, requestMethod, requestTimeout, contentType, requestBody, requestParameters);
}

function getEmailMonitoringBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);

    let mailFolder = figure.userData.mailFolder;
    let markAsRead = figure.userData.markAsRead;
    let filterParameters = figure.userData.filterParameters;

    return new EmailMonitoringTarget(abstractBlock, mailFolder, markAsRead, filterParameters);
}

function getGetKafkaMessageBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);

    let bootstrapServers = figure.userData.bootstrapServers;
    let groupId = figure.userData.groupId;
    let topic = figure.userData.topic;

    return new GetKafkaMessageTarget(abstractBlock, bootstrapServers, groupId, topic);
}

function getGetRabbitMQMessageBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);

    let hostName = figure.userData.hostName;
    let port = figure.userData.port;
    let userName = figure.userData.userName;
    let password = figure.userData.password;
    let virtualHost = figure.userData.virtualHost;
    let queueName = figure.userData.queueName;

    return new GetRabbitMQMessageTarget(abstractBlock, hostName, port, userName, password, virtualHost, queueName);
}

function getQueueTriggerBlockObject(figure, connectors) {
    let abstractBlock = getAbstractBlockObject(figure, connectors);
    let queueName = figure.userData.queueName;
    let reference = figure.userData.reference;
    let priority = figure.userData.priority;
    let comment = figure.userData.comment;
    let queueInputLength = figure.userData.queueLength;
    let queueUpdateDelay = figure.userData.queueUpdateDelay;

    return new QueueTriggerTarget(abstractBlock, queueName, reference, priority, comment, queueInputLength, queueUpdateDelay);
}

function getConnectionObject(connector) {
    let sourcePort = connector.getSource();
    let targetPort = connector.getTarget();

    return new ConnectionTarget(sourcePort.getParent().id, targetPort.getParent().id, getPortObject(sourcePort), getPortObject(targetPort));
}

function getPortObject(port) {
    let portType = null;
    if (port instanceof draw2d.InputPort)
        portType = "In";
    else if (port instanceof draw2d.OutputPort)
        portType = "Out";

    let portLocator = port.getLocator();

    let position = null;

    if (portLocator instanceof draw2d.layout.locator.LeftLocator)
        position = "Left";
    else if (portLocator instanceof draw2d.layout.locator.RightLocator)
        position = "Right";
    else if (portLocator instanceof draw2d.layout.locator.TopLocator || portLocator instanceof TopLocatorWithMargin)
        position = "Top";
    else if (portLocator instanceof draw2d.layout.locator.BottomLocator)
        position = "Bottom";
    else if (portLocator instanceof draw2d.layout.locator.XYRelPortLocator)
        position = "Relative";

    let purpose = "Regular";

    if (port.userData && port.userData.portPurpose)
        purpose = port.userData.portPurpose;

    return new PortTarget(port.getName(), portType, position, purpose);
}

//Инициализация объекта canvas
function initializeCanvas() {
    let canvasObject = new example.Application();
    canvasObject.view.installEditPolicy(new draw2d.policy.canvas.ShowGridEditPolicy());
    canvasObject.view.installEditPolicy(new KeyboardCopyPastePolicy());
    canvasObject.view.installEditPolicy(new draw2d.policy.connection.DragConnectionCreatePolicy({
        createConnection: function () {
            return getConnection();
        }
    }));

    canvasObject.view.fromDocumentToCanvasCoordinate = $.proxy(function (x, y) {
        return new draw2d.geo.Point(
            (x + window.pageXOffset - this.getAbsoluteX() + this.getScrollLeft()) * this.zoomFactor,
            (y + window.pageYOffset - this.getAbsoluteY() + this.getScrollTop()) * this.zoomFactor);
    }, canvasObject.view);

    canvasObject.view.fromCanvasToDocumentCoordinate = $.proxy(function (x, y) {
        return new draw2d.geo.Point(
            ((x * (1 / this.zoomFactor)) + this.getAbsoluteX() - this.getScrollLeft() - window.pageXOffset),
            ((y * (1 / this.zoomFactor)) + this.getAbsoluteY() - this.getScrollTop() - window.pageYOffset));
    }, canvasObject.view);

    canvasObject.view.setCanvasSize(5000, 5000);

    return canvasObject;
}

function getConnection() {
    var routerToUse = new draw2d.layout.connection.SplineConnectionRouter();

    var connection = new draw2d.Connection({
        sourceDecorator: new draw2d.decoration.connection.CircleDecorator(),
        targetDecorator: new draw2d.decoration.connection.ArrowDecorator(),
        stroke: 3,
        color: "#000000",
        router: routerToUse
    });

    connection.installEditPolicy(new HighlightLineSelectionFeedbackPolicy());

    connection.on("removed", () => { validateProcessScheme(null); });

    return connection;
}

/**
 * @method
 * Get ProcessScheme JSON string
 * @param {any} [canvas] draw2d canvas object
 */
function getStageJSON(canvas) {

    let targets = serializeDraw2dStageToCustomJSON(canvas.view);
    let stage = new BlockStageState(targets);

    return JSON.stringify(stage, null, 2);
}

function restoreSchemeFromBlocks(canvasToRestore, schemeJSON) {
    var canvas = canvasToRestore.view;
    var blocks = schemeJSON.Blocks;

    initializeCanvasFromBlocks(blocks, canvas);
}

function initializeCanvasFromBlocks(blockObjects, canvas) {
    for (var blockObject of blockObjects) {
        let canvasBlock;

        if (blockObject.Type === "start") {
            canvasBlock = new StartBlock({ id: blockObject.Id });
        }
        else if (blockObject.Type === "task") {
            canvasBlock = new TaskBlock({ id: blockObject.Id });
            canvasBlock.userData.taskId = blockObject.TaskId;
            canvasBlock.userData.parameters = blockObject.TaskParameters;

            if (blockObject.BlockName && blockObject.BlockName.length > 0)
                setBlockNameLabel(canvasBlock, "taskName", blockObject.BlockName);
            else
                setBlockNameLabel(canvasBlock, "taskName", blockObject.TaskName);

            let relativePorts = blockObject.Ports.filter(el => el.Position === "Relative");
            let portPositionStep = 100.0 / (relativePorts.length + 1);

            let i = 1;

            for (let portObject of relativePorts) {
                let relLocator = new draw2d.layout.locator.XYRelPortLocator(100, portPositionStep * i);
                let createdPort = canvasBlock.createPort("output", relLocator);
                createdPort.setName(portObject.PortName);
                createdPort.setBackgroundColor(portObject.Color);
                createdPort.setMaxFanOut(1);

                i += 1;
            }
        }
        else if (blockObject.Type === "condition") {
            canvasBlock = new ConditionBlock({ id: blockObject.Id });
            canvasBlock.userData.conditionParameters = blockObject.ConditionParameters;

            setBlockNameLabel(canvasBlock, "conditionName", blockObject.BlockName);
        }
        else if (blockObject.Type === "getqueueelement") {
            canvasBlock = new GetQueueElementBlock({ id: blockObject.Id });
            canvasBlock.userData.queueName = blockObject.QueueName;
            canvasBlock.userData.queueElementStatusAfterReceiving = blockObject.QueueElementStatusAfterReceiving;
            canvasBlock.userData.reference = blockObject.Reference;
            canvasBlock.userData.priority = blockObject.Priority;
            canvasBlock.userData.comment = blockObject.Comment;

            setBlockNameLabel(canvasBlock, "getQueueElementName", blockObject.BlockName);
        }
        else if (blockObject.Type === "pushqueueelement") {
            canvasBlock = new PushQueueElementBlock({ id: blockObject.Id });
            canvasBlock.userData.queueName = blockObject.QueueName;
            canvasBlock.userData.value = blockObject.Value;
            canvasBlock.userData.priority = blockObject.Priority;
            canvasBlock.userData.postponeDate = blockObject.PostponeDate;
            canvasBlock.userData.deadlineDate = blockObject.DeadlineDate;
            canvasBlock.userData.comment = blockObject.Comment;
            canvasBlock.userData.reference = blockObject.Reference;
            canvasBlock.userData.isReferenceUnique = blockObject.IsReferenceUnique;

            setBlockNameLabel(canvasBlock, "blockName", blockObject.BlockName);
        }
        else if (blockObject.Type === "parallelexecution") {
            canvasBlock = new ParallelExecutionBlock({ id: blockObject.Id });
        }
        else if (blockObject.Type === "confirmation") {
            canvasBlock = new ConfirmationBlock({ id: blockObject.Id });
            canvasBlock.userData.recipientEmail = blockObject.RecipientEmail;
            canvasBlock.userData.question = ParametersInTextHelper.convertCommonTagsToCKEditorTags(blockObject.Question);
            canvasBlock.userData.emailSubject = blockObject.EmailSubject;
            canvasBlock.userData.answers = blockObject.Answers;

            setBlockNameLabel(canvasBlock, "confirmationName", blockObject.BlockName);

            let outputPorts = blockObject.Ports.filter(el => el.Type === "Out");
            let portPositionStep = 100.0 / (outputPorts.length + 1);

            let i = 1;

            for (let portObject of outputPorts) {
                let relLocator = new draw2d.layout.locator.XYRelPortLocator(portPositionStep * i, 100);
                let createdPort = canvasBlock.createPort("output", relLocator);
                createdPort.setName(portObject.PortName);
                createdPort.setBackgroundColor("#5A99FA");
                createdPort.setMaxFanOut(1);
                createdPort.on("connect", ConfirmationBlockContentHandler.onPortConnected);

                let portLabel = new draw2d.shape.basic.Label({
                    text: i,
                    stroke: 0,
                    bold: true,
                    radius: 0,
                    bgColor: null,
                    fontColor: "#000",
                    resizeable: true,
                    fontSize: 10
                });

                createdPort.add(portLabel, new draw2d.layout.locator.TopLocator());

                i += 1;
            }
        }
        else if (blockObject.Type === "sendemail") {
            canvasBlock = new SendEmailBlock({ id: blockObject.Id });
            canvasBlock.userData.recipientEmail = blockObject.RecipientEmail;
            canvasBlock.userData.emailSubject = blockObject.EmailSubject;
            canvasBlock.userData.emailBody = ParametersInTextHelper.convertCommonTagsToCKEditorTags(blockObject.EmailBody);

            setBlockNameLabel(canvasBlock, "blockName", blockObject.BlockName);
        }
        else if (blockObject.Type === "httprequest") {
            canvasBlock = new HttpRequestBlock({ id: blockObject.Id });
            canvasBlock.userData.requestUri = blockObject.RequestUri;
            canvasBlock.userData.requestMethod = blockObject.RequestMethod;
            canvasBlock.userData.requestTimeout = blockObject.RequestTimeout;
            canvasBlock.userData.contentType = blockObject.ContentType;
            canvasBlock.userData.requestBody = blockObject.RequestBody;
            canvasBlock.userData.requestParameters = blockObject.RequestParameters;

            setBlockNameLabel(canvasBlock, "blockName", blockObject.BlockName);
        }
        else if (blockObject.Type === "emailmonitoring") {
            canvasBlock = new EmailMonitoringBlock({ id: blockObject.Id });
            canvasBlock.userData.mailFolder = blockObject.MailFolder;
            canvasBlock.userData.markAsRead = blockObject.MarkAsRead;
            canvasBlock.userData.filterParameters = blockObject.FilterParameters;

            setBlockNameLabel(canvasBlock, "blockName", blockObject.BlockName);
        }
        else if (blockObject.Type === "getkafkamessage") {
            canvasBlock = new GetKafkaMessageBlock({ id: blockObject.Id });
            canvasBlock.userData.bootstrapServers = blockObject.BootstrapServers;
            canvasBlock.userData.groupId = blockObject.GroupId;
            canvasBlock.userData.topic = blockObject.Topic;

            setBlockNameLabel(canvasBlock, "blockName", blockObject.BlockName);
        }
        else if (blockObject.Type === "getrabbitmqmessage") {
            canvasBlock = new GetRabbitMQMessageBlock({ id: blockObject.Id });
            canvasBlock.userData.hostName = blockObject.HostName;
            canvasBlock.userData.port = blockObject.Port;
            canvasBlock.userData.userName = blockObject.UserName;
            canvasBlock.userData.password = blockObject.Password;
            canvasBlock.userData.virtualHost = blockObject.VirtualHost;
            canvasBlock.userData.queueName = blockObject.QueueName;

            setBlockNameLabel(canvasBlock, "blockName", blockObject.BlockName);
        }

        else if (blockObject.Type === "queuetrigger") {
            canvasBlock = new QueueTriggerBlock({ id: blockObject.Id });
            canvasBlock.userData.queueName = blockObject.QueueName;
            canvasBlock.userData.reference = blockObject.Reference;
            canvasBlock.userData.priority = blockObject.Priority;
            canvasBlock.userData.comment = blockObject.Comment;
            canvasBlock.userData.queueLength = blockObject.QueueLength;
            canvasBlock.userData.queueUpdateDelay = blockObject.QueueUpdateDelay;

            setBlockNameLabel(canvasBlock, "queueTriggerName", blockObject.BlockName);
        }

        canvasBlock.userData.blockType = blockObject.Type;
        canvasBlock.userData.blockName = blockObject.BlockName;

        canvasBlock.x = blockObject.CoordinateX;
        canvasBlock.y = blockObject.CoordinateY;

        let portPositions = ["Bottom", "Left", "Top", "Right"];

        for (let pos of portPositions) {
            let portsByPosition = blockObject.Ports.filter(f => f.Position === pos);
            createPortsByPosition(canvasBlock, portsByPosition, pos);
        }

        canvas.add(canvasBlock);
    }

    for (var blockObject of blockObjects) {
        var connectionObjects = blockObject.Connections;

        if (connectionObjects) {
            for (var connObject of connectionObjects) {
                var sourceBlock = canvas.getFigure(connObject.SourceBlockId);
                var sourcePortName = connObject.SourcePort.PortName ? connObject.SourcePort.PortName : connObject.SourcePort.Position.toLowerCase() + connObject.SourcePort.Type + "_" + connObject.SourceBlockId;
                var sourcePort = sourceBlock.getOutputPort(sourcePortName);

                var targetBlock = canvas.getFigure(connObject.TargetBlockId);
                var targetPortName = connObject.TargetPort.PortName ? connObject.TargetPort.PortName : connObject.TargetPort.Position.toLowerCase() + connObject.TargetPort.Type + "_" + connObject.TargetBlockId;
                var targetPort = targetBlock.getInputPort(targetPortName);

                var connection = getConnection();
                connection.setSource(sourcePort);
                connection.setTarget(targetPort);

                canvas.add(connection);
            }
        }
    }

    var parallelExecutionBlockObjects = blockObjects.filter(function (item) {
        return item.Type === "parallelexecution";
    });

    for (var parExBlockObject of parallelExecutionBlockObjects) {

        let canvasParExBlock = canvas.getFigure(parExBlockObject.Id);

        for (innerBlockId of parExBlockObject.InnerBlockIds) {
            let canvasInnerBlock = canvas.getFigure(innerBlockId);

            canvasParExBlock.assignFigure(canvasInnerBlock);
        }
    }
}

function createPortsByPosition(figure, portsByPosition, portPosition) {
    if (portsByPosition.length === 1) {
        let portDescription = portsByPosition[0];
        let portLocator = null;

        if (portPosition === "Bottom")
            portLocator = new draw2d.layout.locator.BottomLocator();
        else if (portPosition === "Left")
            portLocator = new draw2d.layout.locator.LeftLocator();
        else if (portPosition === "Top") {
            if (figure.userData.blockType === "condition")
                portLocator = new TopLocatorWithMargin({ margin: 12 });
            else
                portLocator = new draw2d.layout.locator.TopLocator();
        }
        else if (portPosition === "Right")
            portLocator = new draw2d.layout.locator.RightLocator();

        let portType = null;

        if (portDescription.Type === "In")
            portType = "input";
        else if (portDescription.Type === "Out")
            portType = "output";

        let createdPort = figure.createPort(portType, portLocator);
        createdPort.setName(portDescription.PortName);
        createdPort.setBackgroundColor(portDescription.Color);

        if (portType === "output")
            createdPort.setMaxFanOut(1);
    }
    else if (portsByPosition.length > 1) {
        let portPositionStep = 100.0 / (portsByPosition.length + 1);

        let i = 1;

        for (let portDescription of portsByPosition) {
            let x = 0;
            let y = 0;

            if (portPosition === "Bottom") {
                x = portPositionStep * i;
                y = 100;
            }
            else if (portPosition === "Left") {
                x = 0;
                y = portPositionStep * i;
            }
            else if (portPosition === "Top") {
                x = portPositionStep * i;
                y = 0;
            }
            else if (portPosition === "Right") {
                x = 100;
                y = portPositionStep * i;
            }

            let portLocator = new draw2d.layout.locator.XYRelPortLocator(x, y);

            let portType = null;

            if (portDescription.Type === "In")
                portType = "input";
            else if (portDescription.Type === "Out")
                portType = "output";

            let createdPort = figure.createPort(portType, portLocator);
            createdPort.setName(portDescription.PortName);
            createdPort.setBackgroundColor(portDescription.Color);

            if (portType === "output")
                createdPort.setMaxFanOut(1);

            i += 1;
        }
    }
}

/**
 * @method Validate process scheme
 * @param {function} [callback] function that is called after successful validation
 */
function validateProcessScheme(callback) {
    let scheme = getStageJSON(canvas);

    let validationResultsCard = document.getElementById("validationErrorsCard");
    let validationErrorsCardBodyElement = document.getElementById("validationErrorsCardBody");
    let validationResultsTableBody = document.getElementById("validationErrorsEntries");
    let canvasRowElement = document.getElementById("canvasRow");

    $.ajax({
        url: "/BPM/ValidateProcessScheme",
        contentType: "application/x-www-form-urlencoded",
        method: "POST",
        data: {
            schemeJSON: scheme
        },
        success: function (response) {
            validationResultsTableBody.innerHTML = "";

            let exclamationMarkImages = canvas.view.getFigures().asArray()
                .map(function (p) { return p.children.data; })
                .reduce(function (a, b) { return a.concat(b); }, [])
                .map(function (p) { return p.figure; })
                .filter(f => f.userData && f.userData.isValidationErrorImage);

            for (let image of exclamationMarkImages)
                canvas.view.remove(image);

            if (response && response.length > 0) {
                for (let validationResult of response) {
                    let errorRow = document.createElement("tr");

                    let blockNameCell = document.createElement("td");
                    blockNameCell.innerHTML = validationResult.blockName;

                    let errorMessageCell = document.createElement("td");
                    errorMessageCell.innerHTML = validationResult.validationMessage;

                    errorRow.appendChild(blockNameCell);
                    errorRow.appendChild(errorMessageCell);

                    validationResultsTableBody.appendChild(errorRow);

                    let blockWithError = canvas.view.getFigure(validationResult.id);

                    let exclamationMarkImage = new draw2d.shape.basic.Image({
                        path: '/Images/BPM/ExclamationMark.svg',
                        width: 15,
                        height: 15,
                        minWidth: 15,
                        minHeight: 15,
                        boundingBox: new draw2d.geo.Rectangle(0, 0, 15, 15),
                    });

                    let tooltipLabel = new draw2d.shape.basic.Label({
                        text: validationResult.validationMessage,
                        bgColor: "#f0f0f0",
                        color: "#000",
                        fontColor: "#000",
                        visible: false
                    });

                    exclamationMarkImage.add(tooltipLabel, new draw2d.layout.locator.RightLocator());

                    exclamationMarkImage.onMouseEnter = function () {
                        let targetLabel = this.children.first().figure;
                        targetLabel.setVisible(true);
                        targetLabel.toFront();
                    };

                    exclamationMarkImage.onMouseLeave = function () {
                        let targetLabel = this.children.first().figure;
                        targetLabel.setVisible(false);
                    };

                    exclamationMarkImage.userData = { isValidationErrorImage: true };
                    blockWithError.add(exclamationMarkImage, new draw2d.layout.locator.XYRelPortLocator(0, 0));
                    blockWithError.repaint();
                    exclamationMarkImage.toFront();
                }

                validationResultsCard.style.display = "block";
                validationErrorsCardBodyElement.style.height = "20vh";
                validationErrorsCardBodyElement.style.display = "block";
                canvasRowElement.style.height = "60vh";

                if (callback)
                    showSnackbar("failure", localizedValidationErrorMessage);
            }
            else {
                validationResultsCard.style.display = "none";
                canvasRowElement.style.height = "85vh";

                if (callback)
                    callback();
            }
        }
    });
}