﻿let ConfirmationBlockContentHandler = {

    confirmationQuestionEditor: null,

    initializeConfirmationModal: function () {

        this.initializeBlockFooter();

        if (!this.confirmationQuestionEditor) {
            ClassicEditor
                .create(document.querySelector("#confirmationQuestion"), {
                    mention: {
                        feeds: [
                            {
                                marker: "$",
                                feed: ParametersInTextHelper.getCKEditorParametersList,
                                itemRenderer: ParametersInTextHelper.getCKEditorTagElement
                            }
                        ]
                    },
                    language: currentLanguage
                })
                .then(newEditor => {
                    ConfirmationBlockContentHandler.confirmationQuestionEditor = newEditor;
                    ParametersInTextHelper.applyCKEditorMentionCustomization(ConfirmationBlockContentHandler.confirmationQuestionEditor);

                    if (selectedBlock && selectedBlock.userData && selectedBlock.userData.question)
                        newEditor.setData(selectedBlock.userData.question);
                })
                .catch(error => {
                    console.error(error);
                });
        }

        let emailSubjectInput = document.getElementById("confirmationEmailSubject");
        ParametersInTextHelper.applyTagify(emailSubjectInput);

        if (selectedBlock && selectedBlock.userData) {

            if (selectedBlock.userData.recipientEmail) {
                let recipientEmailInput = document.getElementById("confirmationRecipientEmail");
                recipientEmailInput.value = selectedBlock.userData.recipientEmail;
            }

            if (selectedBlock.userData.question) {
                let questionInput = document.getElementById("confirmationQuestion");
                questionInput.value = selectedBlock.userData.question;
            }

            if (selectedBlock.userData.emailSubject) {
                emailSubjectInput.value = selectedBlock.userData.emailSubject;
            }
        }

        this.fillConfirmationAnswers("confirmationParameters");

        let confirmationPropertiesElement = document.getElementById("confirmationProperties");

        while (modalBlockParametersElement.lastElementChild) {
            modalBlockParametersElement.lastElementChild.style.display = "none";
            modalParametersContainerElement.appendChild(modalBlockParametersElement.lastElementChild);
        }

        modalBlockParametersElement.appendChild(confirmationPropertiesElement);
        confirmationPropertiesElement.style.display = "block";
    },

    initializeBlockFooter: function () {
        let saveBtn = document.createElement("button");
        saveBtn.className = "modal-blackButton";
        saveBtn.style.width = "50%";
        saveBtn.innerHTML = localizedSaveChangesString;

        saveBtn.onclick = function () {
            modal.style.display = "none";
            ConfirmationBlockContentHandler.saveConfirmationProperties("confirmationParameters");

            validateProcessScheme(null);
        }

        while (modalFooter.firstChild) {
            modalFooter.removeChild(modalFooter.firstChild);
        }
        modalFooter.appendChild(saveBtn);
    },

    onPortConnected: function (emitterPort, eventData) {
        let parentBlock = emitterPort.getParent();
        let labelText = parentBlock.userData.answers[emitterPort.getName()];

        if (labelText.length > 10)
            labelText = labelText.substring(0, 9) + "...";

        let connectionLabel = new draw2d.shape.basic.Label({
            text: labelText,
            bgColor: "#f0f0f0",
            color: "#000",
            fontColor: "#000"
        });

        eventData.connection.add(connectionLabel, new draw2d.layout.locator.ManhattanMidpointLocator());
    },

    saveConfirmationProperties: function (confirmationParametersElementId) {
        let confirmationParametersElement = document.getElementById(confirmationParametersElementId);

        let existingPorts = selectedBlock.getOutputPorts().asArray();

        let answerCount = confirmationParametersElement.children.length;

        let confirmationAnswers = {};

        let portPositionStep = 100.0 / (answerCount + 1);

        let i = 1;

        for (let answerRow of confirmationParametersElement.children) {
            let answerInputs = answerRow.getElementsByTagName("input");
            let answerText = answerInputs[0].value;

            let relLocator = new draw2d.layout.locator.XYRelPortLocator(portPositionStep * i, 100);

            let correlatingPort = existingPorts[i - 1];

            let portName = null;

            if (correlatingPort) {
                portName = correlatingPort.getName();

                correlatingPort.setLocator(relLocator);

                let labelText = answerText;

                if (labelText.length > 10)
                    labelText = labelText.substring(0, 9) + "...";

                let portConnections = correlatingPort.getConnections().data;

                if (portConnections.length > 0) {
                    let existingLabel = portConnections[0].getChildren().first();
                    existingLabel.setText(labelText);
                }
            }
            else {
                let createdPort = selectedBlock.createPort("output", relLocator);
                createdPort.setBackgroundColor("#5A99FA");
                createdPort.setMaxFanOut(1);

                createdPort.on("connect", ConfirmationBlockContentHandler.onPortConnected);

                let portLabel = new draw2d.shape.basic.Label({
                    text: i,
                    stroke: 0,
                    bold: true,
                    radius: 0,
                    bgColor: null,
                    fontColor: "#000",
                    fontSize: 10,
                    resizeable: true
                });

                createdPort.add(portLabel, new draw2d.layout.locator.TopLocator());

                portName = createdPort.getName();
            }

            confirmationAnswers[portName] = answerText;

            i += 1;
        }

        if (answerCount < existingPorts.length) {
            for (let j = answerCount; j < existingPorts.length; j++) {
                selectedBlock.removePort(existingPorts[j]);
            }
        }

        if (this.confirmationQuestionEditor)
            selectedBlock.userData.question = this.confirmationQuestionEditor.getData();
        else
            selectedBlock.userData.question = document.getElementById("confirmationQuestion").value;

        selectedBlock.userData.emailSubject = document.getElementById("confirmationEmailSubject").value;
        selectedBlock.userData.answers = confirmationAnswers;

        selectedBlock.userData.blockName = blockNameInputElement.value;
        selectedBlock.userData.recipientEmail = document.getElementById("confirmationRecipientEmail").value;

        setBlockNameLabel(selectedBlock, "confirmationName", blockNameInputElement.value);
    },

    fillConfirmationAnswers: function (confirmationParametersDivId) {
        let confirmationParametersDiv = document.getElementById(confirmationParametersDivId);

        confirmationParametersDiv.innerHTML = "";

        if (selectedBlock.userData && selectedBlock.userData.answers) {

            let answerEntries = Object.entries(selectedBlock.userData.answers);

            let i = 1;

            for (let [key, value] of answerEntries) {
                let inputDiv = this.formAnswerRow(i, value);
                confirmationParametersDiv.append(inputDiv);

                i += 1;
            }
        }
    },

    addAnswerRow: function (confirmationParametersDivId) {
        let confirmationParametersDiv = document.getElementById(confirmationParametersDivId);
        let answerCount = confirmationParametersDiv.children.length;

        confirmationParametersDiv.appendChild(this.formAnswerRow(answerCount + 1, null));
    },

    formAnswerRow: function (pathNumber, answerText) {
        let answerRow = document.createElement("tr");

        let answerNumberLabel = document.createElement("label");
        answerNumberLabel.textContent = pathNumber;

        let labelCell = document.createElement("td");
        labelCell.appendChild(answerNumberLabel);

        answerRow.appendChild(labelCell);

        let valueInput = document.createElement("input");
        valueInput.type = "text";
        valueInput.className = "parametersInput";

        if (answerText)
            valueInput.value = answerText;

        let valueInputCell = document.createElement("td");
        valueInputCell.appendChild(valueInput);

        answerRow.appendChild(valueInputCell);

        let removeRowButton = document.createElement("input");
        removeRowButton.type = "button";
        removeRowButton.className = "modal-transparentButton";
        removeRowButton.value = "X";
        removeRowButton.addEventListener("click", this.removeAnswerRow);

        let removeRowButtonCell = document.createElement("td");
        removeRowButtonCell.appendChild(removeRowButton);

        answerRow.appendChild(removeRowButtonCell);

        return answerRow;
    },

    removeAnswerRow: function (e) {
        let answerRow = e.target.closest("tr");
        let answersTable = answerRow.parentElement;

        answerRow.remove();

        let i = 1;

        for (let tableRow of answersTable.children) {
            let labelCell = tableRow.firstChild;
            let labelElement = labelCell.firstChild;
            labelElement.textContent = i;

            i += 1;
        }
    }
}