﻿var jqCronInstance = null;
var calendarDatepicker = null;
var data = undefined;

function radioButtonChanged(radioDivId, radioName, labelId, selectId, items, localizedLabels) {

    console.log(items);
    data = items;

    let div = document.getElementById(radioDivId);
    let radios = Array.from(document.getElementsByName(radioName));

    let chosenType = radios.find(item => item.checked).value;

    setScheduledEntity(labelId, selectId, chosenType, localizedLabels);

}

function setScheduledEntity(labelId, selectId, type, localizedLabels) {

    label = document.getElementById(labelId);
    select = document.getElementById(selectId);

    if (type === 'task') {
        label.innerText = localizedLabels["Task"];
        label.setAttribute("for", "TaskID");
        select.setAttribute("data-val", "true");
        select.setAttribute("name", "TaskID");
    }
    else {
        label.innerText = localizedLabels["Process"];
        label.setAttribute("for", "ProcessID");
        select.setAttribute("data-val", "true");
        select.setAttribute("name", "ProcessID");
    }

    setOptions(select);
}

function setOptions(select) {
    // очищаем список
    select.options.length = 0;

    data.forEach((item) => {
        select.options.add(new Option(item.Name, item.Id));
    });
}

function loadJqCRON($, containerId, inputId, defaultCRON, language, textID) {

    if (language === '')
        language = 'en';

    jqCronInstance = $('#' + containerId).jqCron({
        enabled_minute: true,
        multiple_dom: true,
        multiple_month: true,
        multiple_mins: true,
        multiple_dow: true,
        multiple_time_hours: true,
        multiple_time_minutes: true,
        default_period: 'week',
        default_value: defaultCRON,
        bind_to: $('#' + inputId),
        bind_method: {
            set: function ($element, value) {
                $element.val(value);
                cronExressionChanged(textID, inputId, language);
            }
        },
        no_reset_button: false,
        lang: language
    }).jqCronGetInstance();

    let cronElement = document.getElementById(inputId);
    cronElement.value = defaultCRON;
}

function getDescriptionOfCRON(cron, locale) {

    return $.ajax({
        url: "/ScheduleTypes/GetDescriptionOfCRON",
        contentType: "application/x-www-form-urlencoded",
        method: "GET",
        data: {
            cron: cron,
            locale: locale
        },
        success: function (response) {
            console.log(response);
        }
    });
}


function setReadableDescriptionToElement(elementID, text) {

    element = document.getElementById(elementID);

    element.value = text;
}

async function cronExressionChanged(textID, CRONInputID, locale) {

    let currentCRON = document.getElementById(CRONInputID).value;

    currentCRON = (currentCRON === '') ? '* * * * *' : currentCRON;

    if (currentCRON != undefined) {
        let text = await getDescriptionOfCRON(currentCRON, locale);
        setReadableDescriptionToElement(textID, text);
    }
}

function modifyCronByUtcOffset(fromLocal, cronElementId, cronDescriptorElementId) {

    let cronElement = document.getElementById(cronElementId);
    cronElement.disabled = true;

    let cronExpression = cronElement.value;

    let date = new Date();
    let currentTimeZoneOffsetInHours = -date.getTimezoneOffset() / 60;

    $.ajax({
        url: "/ScheduleTypes/ModifyCronByUtcOffset",
        contentType: "application/x-www-form-urlencoded",
        method: "GET",
        async: false,
        data: {
            cronExpression: cronExpression,
            localTimeZoneHourOffset: currentTimeZoneOffsetInHours,
            fromLocal: fromLocal
        },
        success: function (response) {
            cronElement.value = response;
            cronElement.disabled = false;

            cronExressionChanged(cronDescriptorElementId, cronElementId, language);
            jqCronInstance.setCron(response);

            cronElement.value = response;
        }
    });
}

function onScheduleModeChanged(inputElement) {

    let inputValue = inputElement.value;

    if (inputValue == 0) {
        $("#schedulerDesignerGroup").show();
        $("#calendarGroup").hide();
    }
    else if (inputValue == 1) {
        $("#schedulerDesignerGroup").hide();
        $("#calendarGroup").show();
    }
}

function onCalendarFileChanged(event) {
    let selectedFile = event.target.files[0];
    let reader = new FileReader();

    reader.onload = function (event) {
        callReadCsvCalendarFile(selectedFile);
    };

    reader.readAsDataURL(selectedFile);
}

function callReadCsvCalendarFile(file) {
    var formData = new FormData();
    formData.append("csvCalendarData", file);

    $.ajax({
        url: "/ScheduleTypes/ReadCsvCalendarFile",
        contentType: false,
        method: "POST",
        data: formData,
        processData: false,
        success: function (response) {
            calendarDatepicker.datepicker("setDates", response);
        },
        error: function (jqXHR, textStatus, errorThrown) {
            alert("An error occured. File may not be in the right format.");
        }
    });
}
